-- Ammo override base

if SERVER then
   AddCSLuaFile( "shared.lua" )
end

ENT.Type = "anim"
--ENT.Base = ""

-- Override these values
ENT.AmmoForWeapon = nil
ENT.AmmoType = "Pistol"
ENT.AmmoAmount = 1
ENT.AmmoMax = 10
ENT.Model = Model( "models/items/boxsrounds.mdl" )


function ENT:Initialize()
   self:RealInit()
end

-- Some subclasses want to do stuff before/after initing (eg. setting color)
-- Using self.BaseClass gave weird problems, so stuff has been moved into a fn
-- Subclasses can easily call this whenever they want to
function ENT:RealInit()
   self.Entity:SetModel( self.Model )	

   self.Entity:PhysicsInit( SOLID_VPHYSICS )
   self.Entity:SetMoveType( MOVETYPE_VPHYSICS )
   --self.Entity:SetMoveType(MOVETYPE_FLYGRAVITY)
   self.Entity:SetSolid( SOLID_BBOX )

   self.Entity:SetCollisionGroup( COLLISION_GROUP_WEAPON)
   local b = 32
   self.Entity:SetCollisionBounds(Vector(-b, -b, -b), Vector(b,b,b))

   -- The init process is strange and mysterious. This function sometimes
   -- exists, sometimes it doesn't. Presumably init is called multiple
   -- times. Anyway, this needs to be in in order for the Touch stuff to work
   -- properly.
   if self.Entity.SetTrigger then
      self.Entity:SetTrigger(true)
   end

   self.taken = false

   -- this made the ammo get physics'd too early, meaning it would fall
   -- through physics surfaces it was lying on on the client, leading to
   -- inconsistencies
   --	local phys = self.Entity:GetPhysicsObject()
   --	if (phys:IsValid()) then
   --		phys:Wake()
   --	end
end

-- Pseudo-clone of SDK's UTIL_ItemCanBeTouchedByPlayer
-- aims to prevent picking stuff up through fences and stuff
function ENT:PlayerCanPickup(ply)
   local ent = self.Entity
   local phys = ent:GetPhysicsObject()   
   local spos = phys:IsValid() and phys:GetPos() or ent:OBBCenter()
   local epos = ply:GetShootPos() -- equiv to EyePos in SDK

   local tr = util.TraceLine({start=spos, endpos=epos, filter={ply, ent}, mask=MASK_SOLID})

   -- can pickup if trace was not stopped
   return tr.Fraction == 1.0
end

function ENT:CheckForWeapon(ply, weps)
   if type(weps) == "table" then
      for _, w in pairs(weps) do
         if ply:HasWeapon(w) then
            return true
         end
      end
   else
      return ply:HasWeapon(weps)
   end

   return false
end

function ENT:Touch(ent)
   if SERVER and self.taken != true then
      if (ent:IsValid() and ent:IsPlayer() and self:CheckForWeapon(ent, self.AmmoForWeapon) and self:PlayerCanPickup(ent)) then

         local ammo = ent:GetAmmoCount(self.AmmoType)
         -- need clipmax info and room for at least 1/4th
         if self.AmmoMax >= (ammo + math.ceil(self.AmmoAmount * 0.25)) then
            local given = self.AmmoAmount
            given = math.min(given, self.AmmoMax - ammo)
            ent:GiveAmmo( given, self.AmmoType)
            
            self:Remove()
            
            -- just in case remove does not happen soon enough
            self.taken = true
         end
      end
   end
end

-- Hack to force ammo to physwake
if SERVER then
   function ENT:Think()
      if not self.first_think then
         self:PhysWake()
         self.first_think = true

         -- Immediately unhook the Think, save cycles. The first_think thing is
         -- just there in case it still Thinks somehow in the future.
         self.Think = nil
      end
   end
end

