-- c4 explosive


if SERVER then
   AddCSLuaFile("cl_init.lua")
   AddCSLuaFile("shared.lua")
end

if CLIENT then
   -- this entity can be DNA-sampled so we need some display info
   ENT.Icon = "VGUI/ttt/icon_c4"
   ENT.PrintName = "C4"
end

CODELENGTH = 3
MINIMUM_TIME = 45

ENT.Type = "anim"
ENT.Model = Model("models/weapons/w_c4_planted.mdl")

ENT.CanHavePrints = true
ENT.CanUseKey = true

AccessorFunc( ENT, "thrower", "Thrower")

AccessorFunc( ENT, "radius", "Radius", FORCE_NUMBER )
AccessorFunc( ENT, "dmg", "Dmg", FORCE_NUMBER )

AccessorFunc( ENT, "armcode", "Code" )
AccessorFunc( ENT, "arm_time", "ArmTime", FORCE_NUMBER)
AccessorFunc( ENT, "timer_length", "TimerLength", FORCE_NUMBER)

-- Generate accessors for DT vars. This way all consumer code can keep accessing
-- the vars as they always did, the only difference is that behind the scenes
-- they are set up as DT vars.
AccessorFuncDT(ENT, "explode_time", "ExplodeTime")
AccessorFuncDT(ENT, "armed", "Armed")

ENT.Beep = 0

ENT.DetectiveNearRadius = 300

function ENT:SetupDataTables()
   self:DTVar("Int", 0, "explode_time")
   self:DTVar("Bool", 0, "armed")
end

function ENT:Initialize()
   self.Entity:SetModel(self.Model)
   
   if SERVER then
      self.Entity:PhysicsInit(SOLID_VPHYSICS)
   end
   self.Entity:SetMoveType(MOVETYPE_VPHYSICS)
   self.Entity:SetSolid(SOLID_BBOX)
   self.Entity:SetCollisionGroup(COLLISION_GROUP_WEAPON)

   if SERVER then
      self.Entity:SetUseType(SIMPLE_USE)
   end

   self:SetTimerLength(0)
   self:SetExplodeTime(0)
   self:SetArmed(false)
   if not self:GetThrower() then self:SetThrower(nil) end

   if not self:GetRadius() then self:SetRadius(1000) end
   if not self:GetDmg() then self:SetDmg(200) end

end

function ENT:SetDetonateTimer(length)
   self:SetTimerLength(length)
   self:SetExplodeTime( CurTime() + length )
end


--function ENT:Use(activator, caller)
function ENT:UseOverride(activator)
   if ValidEntity(activator) and activator:IsPlayer() then
      local ply = activator

      self:ShowC4Config(ply)
   end
end

function ENT:WeldToGround(state)
   if self.IsOnWall then return end

   if state then
      -- getgroundentity does not work for non-players
      -- so sweep ent downward to find what we're lying on
      local ignore = player.GetAll()
      table.insert(ignore, self.Entity)

      local tr = util.TraceEntity({start=self:GetPos(), endpos=self:GetPos() - Vector(0,0,16), filter=ignore, mask=MASK_SOLID}, self.Entity)

      if tr.Hit and (ValidEntity(tr.Entity) or tr.HitWorld) then
         local phys = self:GetPhysicsObject()
         if ValidEntity(phys) then
            if tr.HitWorld then
               phys:EnableMotion(false)
            else
               self.OrigMass = phys:GetMass()
               phys:SetMass(150)
            end
         end

         -- only weld to objects we cannot pick up
         local entphys = tr.Entity:GetPhysicsObject()
         if ValidEntity(entphys) and entphys:GetMass() > CARRY_WEIGHT_LIMIT then
            constraint.Weld(self, tr.Entity, 0, 0, 0, true)
         end
      end
   else
      constraint.RemoveConstraints(self.Entity, "Weld")
      local phys = self:GetPhysicsObject()
      if ValidEntity(phys) then
         phys:EnableMotion(true)
         phys:SetMass(self.OrigMass or 10)
      end
   end
end

function ENT:SphereDamage(center, radius)
   -- It seems intuitive to use FindInSphere here, but that will find all ents
   -- in the radius, whereas there exist only ~16 players. Hence it is more
   -- efficient to cycle through all those players and do a Lua-side distance
   -- check.

   local r = radius ^ 2 -- square so we can compare with dotproduct directly
   local dmgowner = self:GetThrower()

   -- pre-declare to avoid realloc
   local d = 0.0
   local diff = nil
   local dmg = 0
   for _, ent in pairs(player.GetAll()) do
      if ValidEntity(ent) and ent:Team() == TEAM_TERROR then

         -- dot of the difference with itself is distance squared
         diff = center - ent:GetPos()
         d = diff:DotProduct(diff)

         if d < r then
            -- deadly up to a certain range, then a quick falloff within 100 units
            d = math.max(0, math.sqrt(d) - 490)
            dmg = -0.01 * (d^2) + 125

            local dmginfo = DamageInfo()
            dmginfo:SetDamage(dmg)
            dmginfo:SetAttacker(dmgowner)
            dmginfo:SetInflictor(self.Entity)
            dmginfo:SetDamageType(DMG_BLAST)
            dmginfo:SetDamageForce(center - ent:GetPos())
            dmginfo:SetDamagePosition(ent:GetPos())

            ent:TakeDamageInfo(dmginfo)
         end
      end
   end
end

local c4boom = Sound("c4.explode")
function ENT:Explode(tr)
   if SERVER then
      self.Entity:SetNoDraw(true)
      self.Entity:SetSolid(SOLID_NONE)

      -- pull out of the surface
      if tr.Fraction != 1.0 then
         self.Entity:SetPos(tr.HitPos + tr.HitNormal * 0.6)
      end

      local pos = self.Entity:GetPos()

      if util.PointContents(pos) == CONTENTS_WATER then
         self:Remove()
         return
      end

      -- damage through walls
      self:SphereDamage(pos, 750)

      -- explosion damage
      util.BlastDamage(self, self:GetThrower(), pos, self:GetRadius(), self:GetDmg())

      local effect = EffectData()
      effect:SetStart(pos)
      effect:SetOrigin(pos)
      -- these don't have much effect with the default Explosion
      effect:SetScale(self:GetRadius())
      effect:SetRadius(self:GetRadius())
      effect:SetMagnitude(self:GetDmg())

      if tr.Fraction != 1.0 then
         effect:SetNormal(tr.HitNormal)
      end

      effect:SetOrigin(pos)
      util.Effect("Explosion", effect, true, true) 
      util.Effect("HelicopterMegaBomb", effect, true, true)

      timer.Simple(0.1, function() WorldSound(c4boom, pos, 100, 100) end)

      -- extra push
      local phexp = ents.Create("env_physexplosion")
      phexp:SetPos(pos)
      phexp:SetKeyValue("magnitude", self:GetDmg())
      phexp:SetKeyValue("radius", self:GetRadius())
--      phexp:SetKeyValue("spawnflags", "27")
      phexp:SetKeyValue("spawnflags", "19")
      phexp:Spawn()
      phexp:Fire("Explode", "", 0)


      -- few fire bits to ignite things
      timer.Simple(0.2, function(p) StartFires(pos, tr, 4, 5, true, p) end, self:GetThrower())

      self:SetExplodeTime(0)

      SCORE:HandleC4Explosion(self:GetThrower(), self:GetArmTime(), CurTime())

      self:Remove()
   else
      local spos = self.Entity:GetPos()
      local trs = util.TraceLine({start=spos + Vector(0,0,64), endpos=spos + Vector(0,0,-128), filter=self})
      util.Decal("Scorch", trs.HitPos + trs.HitNormal, trs.HitPos - trs.HitNormal)      

      self:SetExplodeTime(0)
   end
end

function ENT:IsDetectiveNear()
   local center = self:GetPos()
   local r = self.DetectiveNearRadius ^ 2
   local d = 0.0
   local diff = nil
   for _, ent in pairs(player.GetAll()) do
      if IsValid(ent) and ent:IsActiveDetective() then
         -- dot of the difference with itself is distance squared
         diff = center - ent:GetPos()
         d = diff:DotProduct(diff)

         if d < r then
            if ent:HasWeapon("weapon_ttt_defuser") then
               return true
            end
         end
      end
   end

   return false
end

--local beep = Sound("Buttons.snd17")
local beep = Sound("weapons/c4/c4_beep1.wav")
function ENT:Think()

   local etime = self:GetExplodeTime()
   if self:GetArmed() and etime != 0 and etime < CurTime() then
      -- find the ground if it's near and pass it to the explosion
      local spos = self.Entity:GetPos()
      local tr = util.TraceLine({start=spos, endpos=spos + Vector(0,0,-32), mask=MASK_SHOT_HULL, filter=self:GetThrower()})

      local success, err = pcall(self.Explode, self, tr)
      if not success then
         -- prevent effect spam on Lua error
         self:Remove()
         
         ErrorNoHalt("ERROR CAUGHT: ttt_c4: " .. err .. "\n")
      end
   elseif self:GetArmed() and CurTime() > self.Beep then
      local amp = 59

      if self:IsDetectiveNear() then
         amp = 75

         local dlight = CLIENT and DynamicLight(self:EntIndex())
         if dlight then
            dlight.Pos = self:GetPos()
            dlight.r = 255
            dlight.g = 0
            dlight.b = 0
            dlight.Brightness = 1
            dlight.Size = 128
            dlight.Decay = 500
            dlight.DieTime = CurTime() + 0.1
         end
      end

      if SERVER then
         WorldSound(beep, self:GetPos(), amp, 100)
      end
      self.Beep = CurTime() + 1.1 - (1 / (etime - CurTime()))
   end
end

-- Timer configuration handlign

if SERVER then
   -- Inform traitors about us
   function ENT:SendWarn(armed)
      umsg.Start("c4_warn", GetTraitorFilter(true))
      umsg.Short(self.Entity:EntIndex())
      umsg.Bool(armed)
      if armed then
         umsg.Vector(self.Entity:GetPos())
         umsg.Float(self:GetExplodeTime())
      end
      umsg.End()
   end

   function ENT:OnRemove()
      self:SendWarn(false)
   end

   function ENT:Disarm()
      self:SetExplodeTime(0)
      self:SetArmed(false)
      self:WeldToGround(false)
      self:SendWarn(false)
   end

   function ENT:ShowC4Config(ply)
      -- show menu to player to configure or disarm us
      umsg.Start("c4_config", ply)
      umsg.Short(self.Entity:EntIndex())
      umsg.End()
   end

   local function ReceiveC4Config(ply, cmd, args)
      if not ValidEntity(ply) or not ply:IsTerror() or not ply:Alive() or #args != 3 then return end
      local idx = tonumber(args[1])
      local time = tonumber(args[2])
      local code = tostring(args[3])
      
      if not idx or not time or not code then return end

      code = string.gsub(code, "[^%d]", "")

      local bomb = ents.GetByIndex(idx)
      if ValidEntity(bomb) and not bomb:GetArmed()then
         if bomb:GetPos():Distance(ply:GetPos()) > 256 then
            PlayerMsg(ply, string.upper("You are too far from the bomb to arm it."), false)
         elseif time < MINIMUM_TIME or time > 600 or string.len(code) != CODELENGTH then
            PlayerMsg(ply, string.upper("Failed to arm bom due to invalid time or code."), false)
         elseif ValidEntity(bomb:GetPhysicsObject()) and bomb:GetPhysicsObject():HasGameFlag(FVPHYSICS_PLAYER_HELD) then
            PlayerMsg(ply, string.upper("You cannot arm the bomb while carrying it."))
         else
            PlayerMsg(ply, string.upper("You have successfully armed the bomb."), false)

            ply.bomb_code = code
            
            --print("C4 will explode in", time)
            bomb:SetDetonateTimer(time)
            bomb:SetCode(code)
            bomb:SetArmed(true)
            bomb:WeldToGround(true)
            bomb:SetArmTime(CurTime())

            bomb:SendWarn(true)
         end
      end
      
   end
   concommand.Add("ttt_c4_config", ReceiveC4Config)

   local function SendDisarmResult(ply, idx, results)
      umsg.Start("c4_disarm_result", ply)
      umsg.Short(idx)
      for i=1, CODELENGTH do
         umsg.Bool(results[i])
      end
      umsg.End()
   end

   local function ReceiveC4Disarm(ply, cmd, args)
      if not ValidEntity(ply) or not ply:IsTerror() or not ply:Alive() or #args != 2 then return end
      local idx = tonumber(args[1])
      local code = tostring(args[2])
      
      if not idx or not code then return end

      code = string.gsub(code, "[^%d]", "")

      local bomb = ents.GetByIndex(idx)
      if ValidEntity(bomb) and bomb:GetArmed() then
         if bomb:GetCode() == code then
            if bomb:GetPos():Distance(ply:GetPos()) > 256 then
               PlayerMsg(ply, string.upper("You are too far from the bomb to disarm it."), false)
            else
               PlayerMsg(ply, string.upper("You have successfully disarmed the bomb."), false)
               bomb:Disarm()

               -- only case with success umsg
               SendDisarmResult(ply, idx, {true, true, true})
               return
            end
         else
            local attempt = string.ToTable(code)
            local correct = string.ToTable(bomb:GetCode())

            local r = {}
            for k, a in pairs(attempt) do
               r[k] = (a == correct[k])
            end

            if table.Count(r) == CODELENGTH then
               SendDisarmResult(ply, idx, r)
            end

            -- penalise for wrong code if an innocent
            if not ply:GetTraitor() then
               local penalty = math.min(1, bomb:GetTimerLength() / 10)
               bomb:SetExplodeTime(bomb:GetExplodeTime() - penalty)

               bomb:SendWarn(true)
            end
         end
         
         SendDisarmResult(ply, idx, {false, false, false})
      end
   end
   concommand.Add("ttt_c4_disarm", ReceiveC4Disarm)


   local function ReceiveC4Pickup(ply, cmd, args)
      if not ValidEntity(ply) or not ply:IsTerror() or not ply:Alive() or #args != 1 then return end
      local idx = tonumber(args[1])

      if not idx then return end

      local bomb = ents.GetByIndex(idx)
      if ValidEntity(bomb) and not bomb:GetArmed() then
         if bomb:GetPos():Distance(ply:GetPos()) > 256 then
            PlayerMsg(ply, string.upper("You are too far from the bomb to take it."), false)
         elseif ply:HasEquipmentWeapon() then
            PlayerMsg(ply, string.upper("You are already carrying special equipment."), false)
         else
            local prints = bomb.fingerprints or {}
            bomb:Remove()

            local wep = ply:Give("weapon_ttt_c4")
            if IsValid(wep) then
               wep.fingerprints = wep.fingerprints or {}
               table.Add(wep.fingerprints, prints)
            end
            return
         end
      end
   end
   concommand.Add("ttt_c4_pickup", ReceiveC4Pickup)
end

if CLIENT then
   local strtime = string.FormattedTime
   local max = math.max
   function ENT:Draw()
      self.Entity:DrawModel()

      if self:GetArmed() then
         --local angpos_ll = self:GetAttachment(self:LookupAttachment("controlpanel0_ll"))
         local angpos_ur = self:GetAttachment(self:LookupAttachment("controlpanel0_ur"))
         if angpos_ur then
            cam.Start3D2D(angpos_ur.Pos, angpos_ur.Ang, 0.2)
            draw.DrawText(strtime(max(0, self:GetExplodeTime() - CurTime()), "%02i:%02i"), "Default", -1, 1, COLOR_RED, TEXT_ALIGN_RIGHT)
            cam.End3D2D()
         end
      end
   end
end

