
if SERVER then
   
   AddCSLuaFile( "shared.lua" )
end
	
SWEP.HoldType			= "melee"

if CLIENT then

   SWEP.PrintName			= "Knife"
   SWEP.Instructions		= "Cut and tear"
   SWEP.Slot				= 6
   SWEP.SlotPos			= 0
   SWEP.IconLetter			= "u"
  
   SWEP.ViewModelFlip = false

   SWEP.EquipMenuData = {
      type="Weapon",
      desc="Kills instantly and without a sound, but\nonly once.\n\nCan be thrown using alternate fire.\n"
   };

   SWEP.Icon = "VGUI/ttt/icon_knife"
end

SWEP.Base				= "weapon_tttbase"
SWEP.Spawnable			= true
SWEP.AdminSpawnable		= true
SWEP.ViewModel			= "models/weapons/v_knife_t.mdl"
SWEP.WorldModel			= "models/weapons/w_knife_t.mdl"
SWEP.Weight			= 5
SWEP.AutoSwitchTo		= false
SWEP.AutoSwitchFrom		= false
SWEP.DrawCrosshair		= false
SWEP.Primary.ClipSize		= -1
SWEP.Primary.DefaultClip	= -1
SWEP.Primary.Automatic		= true
SWEP.Primary.Delay = 1.25
SWEP.Primary.Ammo		= "none"
SWEP.Secondary.ClipSize		= -1
SWEP.Secondary.DefaultClip	= -1
SWEP.Secondary.Automatic	= true
SWEP.Secondary.Ammo		= "none"
SWEP.Secondary.Delay = 2

SWEP.Kind = WEAPON_EQUIP
SWEP.CanBuy = {ROLE_TRAITOR} -- only traitors can buy
SWEP.LimitedStock = true -- only buyable once
SWEP.WeaponID = AMMO_KNIFE

SWEP.IsSilent = true

function SWEP:PrimaryAttack()
   self.Weapon:SetNextPrimaryFire( CurTime() + self.Primary.Delay )
   self.Weapon:SetNextSecondaryFire( CurTime() + self.Secondary.Delay )

   self.Owner:LagCompensation(true)

   local spos = self.Owner:GetShootPos() - Vector(0,0, 3)
   local sdest = spos + (self.Owner:GetAimVector() * 65)

   local kmins = Vector(1,1,1) * -10
   local kmaxs = Vector(1,1,1) * 10

   local tr = util.TraceHull({start=spos, endpos=sdest, filter=self.Owner, mask=MASK_SHOT_HULL, mins=kmins, maxs=kmaxs})

   -- Hull might hit environment stuff that line does not hit
   if not ValidEntity(tr.Entity) then
      tr = util.TraceLine({start=spos, endpos=sdest, filter=self.Owner, mask=MASK_SHOT_HULL})
   end

   local hitEnt = tr.Entity

   -- effects
   if ValidEntity(hitEnt) then
      self.Weapon:SendWeaponAnim( ACT_VM_HITCENTER )

      local edata = EffectData()
      edata:SetStart(spos)
      edata:SetOrigin(tr.HitPos)
      edata:SetNormal(tr.Normal)
--      edata:SetDamageType(DMG_SLASH)
      edata:SetEntity(hitEnt)

      if hitEnt:IsPlayer() or hitEnt:GetClass() == "prop_ragdoll" then
         util.Effect("BloodImpact", edata)

         --self.Owner:FireBullets({Num=1, Src=spos, Dir=self.Owner:GetAimVector(), Spread=Vector(0,0,0), Tracer=0, Force=1, Damage=0})
      end
   else
      self.Weapon:SendWeaponAnim( ACT_VM_MISSCENTER )
   end

   if SERVER then
      self.Owner:SetAnimation( PLAYER_ATTACK1 )
   end


   if SERVER and tr.Hit and tr.HitNonWorld and ValidEntity(hitEnt) then

      if hitEnt:IsPlayer() then

         local dmg = DamageInfo()
         dmg:SetDamage(1000)
         dmg:SetAttacker(self.Owner)
         dmg:SetInflictor(self.Weapon)
         dmg:SetDamageForce(self.Owner:GetAimVector() * 1500)
         dmg:SetDamagePosition(self.Owner:GetPos())
         dmg:SetDamageType(DMG_SLASH)

         -- now that we use a hull trace, our hitpos is guaranteed to be
         -- terrible, so try to make something of it with a separate trace and
         -- hope our effect_fn trace has more luck

         -- first a straight up line trace to see if we aimed nicely
         local retr = util.TraceLine({start=spos, endpos=sdest, filter=self.Owner, mask=MASK_SHOT_HULL})

         -- if that fails, just trace to worldcenter so we have SOMETHING
         if not retr.Entity == hitEnt then
            local center = hitEnt:LocalToWorld(hitEnt:OBBCenter())
            retr = util.TraceLine({start=spos, endpos=center, filter=self.Owner, mask=MASK_SHOT_HULL})
         end


         -- create knife effect creation fn
         local bone = retr.PhysicsBone
         local pos = retr.HitPos
         local norm = tr.Normal
         local ang = Angle(-28,0,0) + norm:Angle()
         ang:RotateAroundAxis(ang:Right(), -90)
         pos = pos - (ang:Forward() * 7)

         local prints = self.fingerprints
         local ignore = self.Owner

         hitEnt.effect_fn = function(rag)

                               -- we might find a better location
                               local rtr = util.TraceLine({start=pos, endpos=pos + norm * 40, filter=ignore, mask=MASK_SHOT_HULL})

                               if ValidEntity(rtr.Entity) and rtr.Entity == rag then
                                  bone = rtr.PhysicsBone
                                  pos = rtr.HitPos
                                  ang = Angle(-28,0,0) + rtr.Normal:Angle()
                                  ang:RotateAroundAxis(ang:Right(), -90)
                                  pos = pos - (ang:Forward() * 10)
                               end

                               local knife = ents.Create("prop_physics")
                               knife:SetModel("models/weapons/w_knife_t.mdl")
                               knife:SetPos(pos)
                               knife:SetCollisionGroup(COLLISION_GROUP_DEBRIS)
                               knife:SetAngles(ang)
                               knife:SetNWBool("HasPrints", true)
                               knife.fingerprints = prints

                               knife:Spawn()

                               --knife:SetSolid(SOLID_NONE)
                               local phys = knife:GetPhysicsObject()
                               if IsValid(phys) then
                                  phys:EnableCollisions(false)
                               end

                               constraint.Weld(rag, knife, bone, 0, 0, true)
                            end


         -- seems the spos and sdest are purely for effects/forces?
         hitEnt:DispatchTraceAttack(dmg, spos + (self.Owner:GetAimVector() * 3), sdest)

         -- hitEnt appears to die right there, so we could theoretically get to
         -- the ragdoll in here...

         self:Remove()      
      end
   end

   self.Owner:LagCompensation(false)
end

function SWEP:SecondaryAttack()
   self.Weapon:SetNextPrimaryFire( CurTime() + self.Primary.Delay )
   self.Weapon:SetNextSecondaryFire( CurTime() + self.Secondary.Delay )


   self.Weapon:SendWeaponAnim( ACT_VM_MISSCENTER )

   if SERVER then
      self.Owner:SetAnimation( PLAYER_ATTACK1 )

      local ply = self.Owner
      if not ValidEntity(ply) then return end

      local ang = ply:GetAngles()

      if ang.p < 90 then
         ang.p = -10 + ang.p * ((90 + 10) / 90)
      else
         ang.p = 360 - ang.p
         ang.p = -10 + ang.p * -((90 + 10) / 90)
      end

      local vel = math.min(500, (90 - ang.p) * 6)

      local vfw = ang:Forward()
      local vrt = ang:Right()
      
      local src = ply:GetPos() + (ply:Crouching() and ply:GetViewOffsetDucked() or ply:GetViewOffset())

      src = src + (vfw * 1) + (vrt * 3)

      local thr = vfw * vel + ply:GetVelocity()

      local knife_ang = Angle(-28,0,0) + ang
      knife_ang:RotateAroundAxis(knife_ang:Right(), -90)

      local knife = ents.Create("ttt_knife_proj")
      if not ValidEntity(knife) then return end
      knife:SetPos(src)
      knife:SetAngles(knife_ang)

      knife.fingerprints = self.fingerprints

      knife:Spawn()

      knife:SetOwner(ply)

      local phys = knife:GetPhysicsObject()
      if ValidEntity(phys) then
         phys:SetVelocity(thr)
         phys:AddAngleVelocity(Angle(0, 1500, 0))
         phys:Wake()
      end

      self:Remove()
   end
end

function SWEP:OnRemove()
   if CLIENT and ValidEntity(self.Owner) and self.Owner == LocalPlayer() and self.Owner:Alive() then
      RunConsoleCommand("lastinv")
   end
end




