
-- traitor equipment: teleporter

if SERVER then
   AddCSLuaFile( "shared.lua" )

end

SWEP.HoldType = "normal"

if CLIENT then
   SWEP.PrintName			= "Teleporter"
   SWEP.Instructions		= "Zip zap zoop."
   SWEP.Slot				= 6
   SWEP.SlotPos			= 0

   SWEP.ViewModelFOV = 10

   SWEP.EquipMenuData = {
      type="Weapon",
      model="models/props_lab/miniteleport.mdl",
      desc="Teleport to a previously marked spot.\n\nTeleporting makes noise, and the\nnumber of uses is limited."
   };

   SWEP.Icon = "VGUI/ttt/icon_tport"
end

SWEP.Base = "weapon_tttbase"

SWEP.Spawnable          = true
SWEP.AdminSpawnable     = false
SWEP.ViewModel          = "models/weapons/v_crowbar.mdl"
SWEP.WorldModel         = "models/weapons/w_camphone.mdl"
SWEP.AutoSwitchTo       = false
SWEP.AutoSwitchFrom     = false
SWEP.DrawCrosshair      = false
SWEP.ViewModelFlip      = false
SWEP.Primary.ClipSize       = 8
SWEP.Primary.DefaultClip    = 8
SWEP.Primary.ClipMax = 8
SWEP.Primary.Automatic      = false
SWEP.Primary.Ammo       = "GaussEnergy"
SWEP.Primary.Delay = 1.0

SWEP.Secondary.Automatic    = false
SWEP.Secondary.Ammo     = "none"
SWEP.Secondary.Delay = 1.0

SWEP.Kind = WEAPON_EQUIP
SWEP.CanBuy = {ROLE_TRAITOR} -- only traitors can buy
SWEP.WeaponID = AMMO_TELEPORT

-- This is traitor equipment


SWEP.AllowDrop = true

SWEP.NoSights = true

function SWEP:SetTeleportMark(pos, ang)
   self.teleport = {pos = pos, ang = ang}
end

function SWEP:GetTeleportMark() return self.teleport end

function SWEP:PrimaryAttack()
   self.Weapon:SetNextPrimaryFire( CurTime() + self.Primary.Delay )

   if SERVER then
      self:TeleportStore()
   else
      surface.PlaySound("ui/buttonrollover.wav")
   end
end
function SWEP:SecondaryAttack()
   self.Weapon:SetNextSecondaryFire( CurTime() + self.Secondary.Delay )

   if self.Weapon:Clip1() <= 0 then
      self:DryFire(self.SetNextSecondaryFire)
      return
   end

   -- Disallow initiating teleports during post, as it will occur across the
   -- restart and allow the user an advantage during prep
   if GetRoundState() == ROUND_POST then return end

   if SERVER then
      self:TeleportRecall()
   else
      surface.PlaySound("buttons/combine_button7.wav")
   end
end

local zap = Sound("ambient/levels/labs/electric_explosion4.wav")
local unzap = Sound("ambient/levels/labs/electric_explosion2.wav")

local function Telefrag(victim, attacker)
   if not IsValid(victim) then return end

   local dmginfo = DamageInfo()
   dmginfo:SetDamage(5000)
   dmginfo:SetDamageType(DMG_SONIC)
   dmginfo:SetAttacker(attacker)
   dmginfo:SetDamageForce(Vector(0,0,10))
   dmginfo:SetDamagePosition(attacker:GetPos())

   victim:TakeDamageInfo(dmginfo)
end


local function ShouldCollide(ent)
   local g = ent:GetCollisionGroup()
   return (g != COLLISION_GROUP_WEAPON and
           g != COLLISION_GROUP_DEBRIS and
           g != COLLISION_GROUP_INTERACTIVE_DEBRIS)
end

local function DoTeleport(ply, teleport)
   if IsValid(ply) and ply:IsTerror() and teleport then
      local oldpos = ply:GetPos()
      local pos = teleport.pos

      -- print decal on destination
      util.PaintDown(pos + Vector(0,0,25), "GlassBreak", ply)

      -- perform teleport
      ply:SetPos(pos)
      ply:SetEyeAngles(teleport.ang)
      ply:Freeze(false)

      WorldSound(zap, oldpos, 75, 100)
      WorldSound(unzap, pos, 65, 100)

      -- print decal on source now that we're gone, because else it will refuse
      -- to draw for some reason
      util.PaintDown(oldpos + Vector(0,0,25), "GlassBreak", ply)

      -- first check if we can teleport here at all, because any solid object or
      -- brush will make us stuck and therefore kills us instead, so the trace
      -- checks for anything solid to players that isn't a player
      local tr = nil
      local tres = {start=pos, endpos=pos, mask=MASK_PLAYERSOLID, filter=player.GetAll()}
      local collide = false

      -- This thing is unnecessary if we can supply a collision group to trace
      -- functions, like we can in source and sanity suggests we should be able
      -- to do so, but I have not found a way to do so yet. Until then, re-trace
      -- while extending our filter whenever we hit something we don't want to
      -- hit (like weapons or ragdolls).
      repeat
         tr = util.TraceEntity(tres, ply)

         if tr.HitWorld then
            collide = true
         elseif IsValid(tr.Entity) then
            if ShouldCollide(tr.Entity) then
               collide = true
            else
               table.insert(tres.filter, tr.Entity)
            end
         end
      until (not tr.Hit) or collide

      if collide then
         Telefrag(ply, ply)
      else

         -- find all players in the place where we will be and telefrag them
         local blockers = ents.FindInBox(pos + Vector(-16, -16, 0),
                                         pos + Vector(16, 16, 64))

         for _, block in pairs(blockers) do
            if IsValid(block) then
               if block:IsPlayer() and block != ply then
                  if block:IsTerror() and block:Alive() then
                     -- telefrag blocker
                     Telefrag(block, ply)
                  end
               end
            end
         end

      end
   elseif IsValid(ply) then
      -- should never happen, but at least unfreeze
      ply:Freeze(false)

      TraitorMsg(ply, "TELEPORT FAILED.")
   end
end

local function StartTeleport(ply, teleport)
   if (not IsValid(ply)) or (not ply:IsTerror()) or (not teleport) then
      return end

   local delay = 3

   timer.Simple(delay, DoTeleport, ply, teleport)

   local edata_up = EffectData()
   edata_up:SetOrigin(ply:GetPos())
   edata_up:SetAngle(ply:GetAngles())
   edata_up:SetEntity(ply)
   edata_up:SetMagnitude(delay)

   util.Effect("teleport_beamup", edata_up)

   local edata_dn = EffectData()
   edata_up:SetOrigin(teleport.pos)
   local ang = teleport.ang
   ang = Angle(0, ang.y, ang.r) -- deep copy
   edata_up:SetAngle(ang)
   edata_up:SetEntity(ply)
   edata_up:SetMagnitude(delay)

   util.Effect("teleport_beamdown", edata_dn)
end

function SWEP:TeleportRecall(ply)
   local ply = self.Owner
   if ValidEntity(ply) and ply:IsTerror() then
      if self:GetTeleportMark() then
         
         local g = ply:GetGroundEntity()
         if g != GetWorldEntity() and not IsValid(g) then
            ply:ChatPrint("Cannot teleport unless standing on solid ground!")
            return
         end

         if ply:Crouching() then
            ply:ChatPrint("Cannot teleport while crouched!")
            return
         end

         ply:Freeze(true)

         self:TakePrimaryAmmo(1)

         timer.Simple(0.5, StartTeleport, ply, self:GetTeleportMark())
      else
         TraitorMsg(ply, "NO LOCATION MARKED. MARK A DESTINATION BEFORE TELEPORTING.")
      end
   end
end


function SWEP:TeleportStore()
   local ply = self.Owner
   if ValidEntity(ply) and ply:IsTerror() then

      local g = ply:GetGroundEntity()
      if g != GetWorldEntity() and not IsValid(g) then
         ply:ChatPrint("Cannot mark a teleport location unless standing on solid ground!")
         return
      end

      if ply:Crouching() then
         ply:ChatPrint("Cannot mark a teleport location while crouched!")
         return
      end


      self:SetTeleportMark(ply:GetPos(), ply:EyeAngles())

      TraitorMsg(ply, "TELEPORT LOCATION MARKED.")
   end   
end



function SWEP:Reload()
   return false
end


if CLIENT then
   local hudtxt = {
      {text="Primary fire to mark current location.", font="TabLarge", xalign=TEXT_ALIGN_RIGHT},
      {text="Secondary fire to teleport.", font="TabLarge", xalign=TEXT_ALIGN_RIGHT}
   };
   function SWEP:DrawHUD()
      local x = ScrW() - 80
      hudtxt[1].pos = {x, ScrH() - 80}
      draw.Text(hudtxt[1])
      draw.TextShadow(hudtxt[1], 2)

      hudtxt[2].pos = {x, ScrH() - 60}
      draw.Text(hudtxt[2])
      draw.TextShadow(hudtxt[2], 2)
   end
end

-- Invisible, same hacks as holstered weapon
local hidden = false
function SWEP:Deploy()
   hidden = false
   return true
end

--function SWEP:DrawWorldModel()
--end

--function SWEP:DrawWorldModelTranslucent()
--end

-- not able to do DrawModel stuff in Deploy, so here's a hack
function SWEP:Think()
   if SERVER and not hidden and ValidEntity(self.Owner) and self.Owner:GetActiveWeapon() == self.Weapon then
      self.Owner:DrawViewModel(false)
--      self.Owner:DrawWorldModel(false)
      hidden = true
   end
end
