
if SERVER then
   AddCSLuaFile( "shared.lua" )
end

   
SWEP.HoldType			= "normal"

if CLIENT then
   SWEP.PrintName			= "DNA scanner"
   SWEP.Instructions		= ""
   SWEP.Slot				= 6

   SWEP.ViewModelFOV = 10

   SWEP.EquipMenuData = {
      type="Weapon",
      desc="Collect DNA samples from items\nand use them to find the DNA's owner.\n\nUse on fresh corpses to get the killer's DNA\nand track him down."
   };

   SWEP.Icon = "VGUI/ttt/icon_wtester"
end

SWEP.Base				= "weapon_tttbase"
SWEP.Spawnable			= true
SWEP.AdminSpawnable		= true
SWEP.ViewModel			= "models/weapons/v_crowbar.mdl"
SWEP.WorldModel			= "models/props_lab/huladoll.mdl"
SWEP.Weight			= 5
SWEP.AutoSwitchTo		= false
SWEP.AutoSwitchFrom		= false
SWEP.DrawCrosshair		= false
SWEP.Primary.ClipSize		= -1
SWEP.Primary.DefaultClip	= -1
SWEP.Primary.Automatic		= false
SWEP.Primary.Delay = 1
SWEP.Primary.Ammo		= "none"
SWEP.Secondary.ClipSize		= -1
SWEP.Secondary.DefaultClip	= -1
SWEP.Secondary.Automatic	= false
SWEP.Secondary.Ammo		= "none"
SWEP.Secondary.Delay = 2

SWEP.Kind = WEAPON_EQUIP
SWEP.CanBuy = {ROLE_DETECTIVE} -- only detectives can buy
SWEP.WeaponID = AMMO_WTESTER
SWEP.LimitedStock = true

SWEP.AllowDrop = true

SWEP.NoSights = true

SWEP.Range = 175

SWEP.ItemSamples = {}

SWEP.ScanningSample = nil
SWEP.NowRepeating = nil

local MAX_ITEM = 30
SWEP.MaxItemSamples = MAX_ITEM

local CHARGE_DELAY = 0.1
local CHARGE_RATE = 3
local MAX_CHARGE = 1250

AccessorFuncDT(SWEP, "charge", "Charge")
AccessorFuncDT(SWEP, "repeating", "Repeating")

SWEP.NextCharge = 0

function SWEP:SetupDataTables()
   self:DTVar("Int", 0, "charge")
   self:DTVar("Bool", 0, "repeating")
end

function SWEP:Initialize()
   self:SetCharge(MAX_CHARGE)
   self:SetRepeating(false)

   return self.BaseClass.Initialize(self)
end

local beep_miss = Sound("player/suit_denydevice.wav")
function SWEP:PrimaryAttack()
   self.Weapon:SetNextPrimaryFire( CurTime() + self.Primary.Delay )

   -- will be tracing against players
   self.Owner:LagCompensation(true)

   local spos = self.Owner:GetShootPos()
   local sdest = spos + (self.Owner:GetAimVector() * self.Range)

   local tr = util.TraceLine({start=spos, endpos=sdest, filter=self.Owner, mask=MASK_SHOT})
   local ent = tr.Entity
   if IsValid(ent) and (not ent:IsPlayer()) then
      if SERVER then
         if ent:IsPlayer() then
            --self:GatherPlayerSample(ent)
         elseif ent:GetClass() == "prop_ragdoll" and ent.killer_sample then
            self:GatherRagdollSample(ent)
         elseif ent.fingerprints and #ent.fingerprints > 0 then
            self:GatherObjectSample(ent)
         else
            self:Report("No DNA sample found on target.")
         end
      end
   else
      if CLIENT then
         self.Owner:EmitSound(beep_miss)
      end
   end

   self.Owner:LagCompensation(false)
end

function SWEP:GatherPlayerSample(ent)
   local added = self:AddPlayerSample(ent)
   
   if not added then
      self:Report("Storage limit for human samples reached. Remove old human samples to add new ones.")
   else
      self:Report("Collected DNA sample from " .. ent:Nick() .. ".")
   end
end

function SWEP:GatherRagdollSample(ent)
   local sample = ent.killer_sample
   local ply = sample.killer

   if IsValid(ply) then
      if sample.t < CurTime() then
         self:Report("DNA sample of the killer has decayed.")
         return
      end

      local added = self:AddPlayerSample(ent, ply)
      
      if not added then
         self:Report("Storage limit for samples reached. Remove old samples to add new ones.")
      else
         self:Report("Collected a sample of the killer's DNA from the corpse!")
      end 
   else
      self:Report("No fresh DNA sample found.")
   end
end

function SWEP:GatherObjectSample(ent)
   if ent:GetClass() == "ttt_c4" and ent:GetArmed() then
      self:Report("This bomb is live! Disarm it first!")
   else
      local collected, old, own = self:AddItemSample(ent)

      if collected == -1 then
         self:Report("Storage limit for collected item samples reached. Remove old samples to add new ones.")
      else
         local txt = "Collected " .. collected .. " new DNA sample" .. (collected == 1 and "" or "s") .. " from the object."

         local ignored = old + own
         if ignored > 0 then
            txt = txt .. " Ignored " .. ignored .. " sample".. (ignored == 1 and "" or "s") ..": already collected or your own DNA."
         end

         self:Report(txt)
      end
   end
end

function SWEP:Report(msg)
   TraitorMsg(self.Owner, string.upper(msg))
end

function SWEP:AddPlayerSample(corpse, killer)
   if #self.ItemSamples < self.MaxItemSamples then
      local prnt = {source=corpse, ply=killer}
      if not table.HasTable(self.ItemSamples, prnt) then
         table.insert(self.ItemSamples, prnt)
      end
      return true
   end
   return false
end

function SWEP:AddItemSample(ent)
   if #self.ItemSamples < self.MaxItemSamples then
      table.Shuffle(ent.fingerprints)

      local new = 0
      local old = 0
      local own = 0
      for _, p in pairs(ent.fingerprints) do
         local prnt = {source=ent, ply=p}

         if p == self.Owner then
            own = own + 1
         elseif table.HasTable(self.ItemSamples, prnt) then
            old = old + 1
         else
            table.insert(self.ItemSamples, prnt)

            new = new + 1
         end
      end
      return new, old, own
   end
   return -1
end

function SWEP:RemovePlayerSample(idx)
   if self.PlayerSamples[idx] then
      table.remove(self.PlayerSamples, idx)
      
      self:SendPrints(false)
   end
end

function SWEP:RemoveItemSample(idx)
   if self.ItemSamples[idx] then
      if self.ScanningSample == idx then
         self:ClearScanState()
      end

      table.remove(self.ItemSamples, idx)
      
      self:SendPrints(false)
   end   
end

function SWEP:SecondaryAttack()
   self.Weapon:SetNextSecondaryFire( CurTime() + 0.05 )

   if CLIENT then return end

   self:SendPrints(true)
end

if SERVER then
   -- Sending this all in one umsg limits the max number of samples. 17 player
   -- samples and 20 item samples (with 20 matches) has been verified as
   -- working in the old DNA sampler.
   function SWEP:SendPrints(should_open)
      umsg.Start("showprints", self.Owner)
      umsg.Bool(should_open)

      umsg.Char(#self.ItemSamples)
      for k,v in ipairs(self.ItemSamples) do
         umsg.Entity(v.source)
      end

      umsg.End()
   end

   function SWEP:SendScan(pos)
      local clear = (pos == nil) or (not IsValid(self.Owner))
      umsg.Start("scanresult", self.Owner)
      umsg.Bool(clear)
      if not clear then
         umsg.Vector(pos)
      end
      umsg.End()
   end

   function SWEP:ClearScanState()
      self.ScanningSample = nil
      self.NowRepeating = nil
      self:SendScan(nil)
   end

   function SWEP:PerformScan(idx, repeated)
      if self:GetCharge() < MAX_CHARGE then return end

      local sample = self.ItemSamples[idx]
      if (not sample) or (not IsValid(self.Owner)) then
         if repeated then self:ClearScanState() end
         return
      end

      local target = sample.ply
      if not IsValid(target) then
         self:Report("DNA not detected in area. (Player disconnected?)")
         self:SetCharge(self:GetCharge() - 50)

         if repeated then self:ClearScanState() end
         return
      end

      -- dead targets fall back to their corpse
      if not target:IsTerror() then
         if IsValid(target.server_ragdoll) then
            target = target.server_ragdoll
         end
      end

      local pos = target:LocalToWorld(target:OBBCenter())

      self:SendScan(pos)
      
      self.ScanningSample = idx
      self.NowRepeating = self:GetRepeating()

      local dist = math.ceil(self.Owner:GetPos():Distance(pos))

      self:SetCharge(math.max(0, self:GetCharge() - math.max(50, dist / 2)))
   end

   function SWEP:Think()
      if self:GetCharge() < MAX_CHARGE then
         if self.NextCharge < CurTime() then
            self:SetCharge(math.min(MAX_CHARGE, self:GetCharge() + CHARGE_RATE))

            self.NextCharge = CurTime() + CHARGE_DELAY
         end
      elseif self.NowRepeating then
         self:PerformScan(self.ScanningSample, true)
      end

      if not hidden and ValidEntity(self.Owner) and self.Owner:GetActiveWeapon() == self.Weapon then
         self.Owner:DrawViewModel(false)
         self.Owner:DrawWorldModel(false)
         hidden = true
      end
   end
end

-- Helper to get at a player's scanner, if he has one
local function GetTester(ply)
   if IsValid(ply) then
      local tester = ply:GetActiveWeapon()
      if IsValid(tester) and tester:GetClass() == "weapon_ttt_wtester" then
         return tester
      end
   end
   return nil
end


if CLIENT then
   local hudtxt = {
      {text="Primary fire to collect a sample from target.", font="TabLarge", xalign=TEXT_ALIGN_RIGHT},
      {text="Secondary fire to open scan menu.", font="TabLarge", xalign=TEXT_ALIGN_RIGHT}
   };
   function SWEP:DrawHUD()
      local x = ScrW() - 80
      hudtxt[1].pos = {x, ScrH() - 80}
      draw.Text(hudtxt[1])
      draw.TextShadow(hudtxt[1], 2)

      hudtxt[2].pos = {x, ScrH() - 60}
      draw.Text(hudtxt[2])
      draw.TextShadow(hudtxt[2], 2)

      local spos = self.Owner:GetShootPos()
      local sdest = spos + (self.Owner:GetAimVector() * self.Range)

      local tr = util.TraceLine({start=spos, endpos=sdest, filter=self.Owner, mask=MASK_SHOT})

      local length = 20
      local gap = 5

      local can_sample = false

      local ent = tr.Entity
      if IsValid(ent) then
         -- weapon or dropped equipment
         if ((tr.Entity:IsWeapon() or tr.Entity.CanHavePrints) or
             -- knife in corpse, or a ragdoll
             ent:GetNWBool("HasPrints", false) or ent:GetNWString("nick", false)) then

            surface.SetDrawColor(0, 255, 0, 255)
            gap = 0

            can_sample = true
         else
            surface.SetDrawColor(255, 0, 0, 200)
            gap = 0
         end
      else
         surface.SetDrawColor(255, 255, 255, 200)
      end

      local x = ScrW() / 2.0
      local y = ScrH() / 2.0

      surface.DrawLine( x - length, y, x - gap, y )
      surface.DrawLine( x + length, y, x + gap, y )
      surface.DrawLine( x, y - length, x, y - gap )
      surface.DrawLine( x, y + length, x, y + gap )

      if ent and can_sample then
         surface.SetFont("DefaultFixedDropShadow")
         surface.SetTextColor(0, 255, 0, 255)
         surface.SetTextPos( x + length*2, y - length*2 )
         surface.DrawText("TYPE: " .. (ent.player_ragdoll and "BODY" or "ITEM"))
         surface.SetTextPos( x + length*2, y - length*2 + 15)
         surface.DrawText("ID:   #" .. ent:EntIndex())
      end
   end


   local basedir = "VGUI/ttt/icon_"
   local function ShowPrintsPopup(item_prints, tester)
      local m = 10
      local bw, bh = 100, 25

      local dpanel = vgui.Create("DFrame")
      local w, h = 400, 250
      dpanel:SetSize(w, h)

      dpanel:AlignRight(5)
      dpanel:AlignBottom(5)

      dpanel:SetTitle("DNA scanning")
      dpanel:SetVisible(true)
      dpanel:ShowCloseButton(true)
      dpanel:SetMouseInputEnabled(true)

      local wrap = vgui.Create("DPanel", dpanel)
      wrap:StretchToParent(m/2, m + 15, m/2, m + bh)
      wrap:SetPaintBackground(false)

      -- item sample listing
      local ilist = vgui.Create("DPanelSelect", wrap)
      ilist:StretchToParent(0,0,0,0)
      ilist:EnableHorizontal(true)
      ilist:SetSpacing(1)
      ilist:SetPadding(1)

      if ilist.VBar then 
         ilist.VBar:Remove()
         ilist.VBar = nil
      end

      local iscroll = vgui.Create("DHorizontalScroller", ilist)
      iscroll:SetPos(3,1)
      iscroll:SetSize(363, 66)
      iscroll:SetOverlap(1)

      iscroll.LoadFrom = function(s, tbl, layout)
                            ilist:Clear(true)
                            ilist.SelectedPanel = nil

                            -- Scroller has no Clear()
                            for k, pnl in pairs(s.Panels) do
                               if IsValid(pnl) then
                                  pnl:Remove()
                               end
                            end

                            s.Panels = {}

                            for k, v in ipairs(tbl) do
                               local ic = vgui.Create("SimpleIcon", ilist)

                               ic:SetIconSize(64)

                               local img = v.Icon or (basedir .. "nades")
                               local name = v.PrintName

                               if v:GetClass() == "prop_ragdoll" then
                                  img = basedir .. "corpse"
                                  name = v:GetNWString("nick")
                               end

                               ic:SetIcon(img)

                               local tip = "DNA sample found on " .. (name or "???") .. " (ID #" .. v:EntIndex() .. ")."

                               ic:SetTooltip(tip)

                               ic.key = k
                               ic.val = v

                               if layout then
                                  ic:PerformLayout()
                               end

                               ilist:AddPanel(ic)
                               s:AddPanel(ic)
                            end

                            iscroll:InvalidateLayout()
                         end

      iscroll:LoadFrom(item_prints)

      local delwrap = vgui.Create("DPanel", wrap)
      delwrap:SetPos(m, 70)
      delwrap:SetSize(370, bh)
      delwrap:SetPaintBackground(false)

      local delitem = vgui.Create("DButton", delwrap)
      delitem:SetPos(0,0)
      delitem:SetSize(bw, bh)
      delitem:SetText("Remove selected")
      delitem.DoClick = function()
                           if IsValid(ilist) and IsValid(ilist.SelectedPanel) then
                              local idx = ilist.SelectedPanel.key
                              RunConsoleCommand("ttt_wtester_remove", idx)
                           end
                        end

      delitem.Think = function(s)
                         if IsValid(ilist) and IsValid(ilist.SelectedPanel) then
                            s:SetEnabled(true)
                         else
                            s:SetEnabled(false)
                         end
                      end

      local delhlp = vgui.Create("DLabel", delwrap)
      delhlp:SetPos(bw + m, 0)
      delhlp:SetText("These are DNA samples collected from items.\nMax. " .. MAX_ITEM .. " item samples can be stored at one time.")
      delhlp:SizeToContents()


      -- hammer out layouts
      wrap:PerformLayout()
      -- scroller needs to sort itself out so it displays all icons it should
      iscroll:PerformLayout()

      local mwrap = vgui.Create("DPanel", wrap)
      mwrap:SetPaintBackground(false)
      mwrap:SetPos(m,100)
      mwrap:SetSize(370, 90)

      
      local bar = vgui.Create("DProgressBar", mwrap)
      bar:SetSize(370, 35)
      bar:SetPos(0, 0)
      bar:CenterHorizontal()
      bar:SetMin(0)
      bar:SetMax(MAX_CHARGE)
      bar:SetValue(tester and math.min(MAX_CHARGE, tester:GetCharge()))
      bar:SetColor(COLOR_GREEN)
      bar:LabelAsPercentage()

      local state = vgui.Create("DLabel", bar)
      state:SetSize(0, 35)
      state:SetPos(10, 6)
      state:SetFont("Trebuchet22")
      state:SetText("READY")
      state:SetTextColor(COLOR_WHITE)
      state:SizeToContents()

      local scan = vgui.Create("DButton", mwrap)
      scan:SetText("Scan")
      scan:SetSize(bw, bh)
      scan:SetPos(0, 40)
      scan:SetEnabled(false)

      scan.DoClick = function(s)
                         if IsValid(ilist) then
                            local i = ilist.SelectedPanel
                            if IsValid(i) then
                               RunConsoleCommand("ttt_wtester_scan", i.key)
                            end
                         end
                      end

      local dcheck = vgui.Create("DCheckBoxLabel", mwrap)
      dcheck:SetPos(0, 70)
      dcheck:SetText("Auto-repeat")
      dcheck:SetIndent(7)
      dcheck:SizeToContents()
      dcheck:SetValue(tester and tester:GetRepeating())
      dcheck.OnChange = function(s, val)
                           RunConsoleCommand("ttt_wtester_repeat", tostring((val == true) and 1 or 0))
                        end

      local scanhlp = vgui.Create("DLabel", mwrap)
      scanhlp:SetPos(bw + m, 40)
      scanhlp:SetText("When charged, you can scan for the location of\nthe player the selected DNA sample belongs to.\nFinding distant targets drains more energy.")
      scanhlp:SizeToContents()

      -- CLOSE
      local dbut = vgui.Create("DButton", dpanel)
      dbut:SetSize(bw, bh)
      dbut:SetPos(m, h - bh - m/1.5)
      dbut:CenterHorizontal()
      dbut:SetText("Close")
      dbut.DoClick = function() dpanel:Close() end

      dpanel:MakePopup()
      dpanel:SetKeyboardInputEnabled(false)

      -- Expose updating fns
      dpanel.UpdatePrints = function(s, its)
                               if IsValid(iscroll) then
                                  iscroll:LoadFrom(its)
                               end
                            end

      dpanel.Think = function(s)
                        if IsValid(bar) and IsValid(scan) and tester then
                           local charge = tester:GetCharge()
                           bar:SetValue(math.min(MAX_CHARGE, charge))
                           if charge < MAX_CHARGE then
                              bar:SetColor(COLOR_RED)

                              state:SetText("CHARGING")
                              state:SizeToContents()

                              scan:SetEnabled(false)
                           else
                              bar:SetColor(COLOR_GREEN)

                              if IsValid(ilist) and IsValid(ilist.SelectedPanel) then
                                 scan:SetEnabled(true)

                                 state:SetText("READY")
                                 state:SizeToContents()
                              else
                                 state:SetText("SELECT SAMPLE")
                                 state:SizeToContents()
                                 scan:SetEnabled(false)
                              end
                           end
                        end
                     end

      return dpanel
   end

   local printspanel = nil
   local function RecvPrints(um)
      local should_open = um:ReadBool()

      local num = um:ReadChar()
      local item_prints = {}
      for i=1, num do
         table.insert(item_prints, um:ReadEntity())
      end

      if should_open then
         if ValidPanel(printspanel) then
            printspanel:Remove()
         end

         local tester = GetTester(LocalPlayer())

         printspanel = ShowPrintsPopup(item_prints, tester)
      else
         if ValidPanel(printspanel) then
            printspanel:UpdatePrints(item_prints)
         end
      end
   end
   usermessage.Hook("showprints", RecvPrints)

   local beep_success = Sound("buttons/blip2.wav")
   --local beep_fail = Sound("buttons/button11.wav")
   local function RecvScan(um)
      local clear = um:ReadBool()
      if clear then
         RADAR.samples = {}
         RADAR.samples_count = 0
         return
      end

      local target_pos = um:ReadVector()
      if not target_pos then return end

      RADAR.samples = {
         {pos = target_pos}
      };

      RADAR.samples_count = 1

      surface.PlaySound(beep_success)
   end
   usermessage.Hook("scanresult", RecvScan)

else -- SERVER

   local function ScanPrint(ply, cmd, args)
      if #args != 1 then return end

      local tester = GetTester(ply)
      if IsValid(tester) then
         local i = tonumber(args[1])

         if i then
            tester:PerformScan(i)
         end
      end
   end
   concommand.Add("ttt_wtester_scan", ScanPrint)

   local function RemoveSample(ply, cmd, args)
      if #args != 1 then return end

      local idx = tonumber(args[1])
      if not idx then return end

      local tester = GetTester(ply)
      if IsValid(tester) then
         tester:RemoveItemSample(idx)
      end
   end
   concommand.Add("ttt_wtester_remove", RemoveSample)

   local function RepeatedScanning(ply, cmd, args)
      if #args != 1 then return end

      local tester = GetTester(ply)
      if IsValid(tester) then
         local s = tobool(args[1])

         tester:SetRepeating(s)

         tester.NowRepeating = false
      end
   end
   concommand.Add("ttt_wtester_repeat", RepeatedScanning)
end


function SWEP:OnDrop()
   hidden = false
end

function SWEP:Reload()
   return false
end

local hidden = false
function SWEP:Deploy()
   hidden = false
   return true
end

