---- Traitor equipment menu

-- If an equipment item in the big list below specifies a material, it will be
-- loaded from this directory. It is possible to have items that do not have a
-- special icon, in which case they must have a model set so that a sandbox
-- spawn menu style icon can be used.
local mat_dir = "VGUI/ttt/"

-- Previously all buyable things were specified in a big table here. Now all
-- weapons have their information set in their SWEP table, which is pulled in
-- here. All non-weapon items are still here.

-- stick to around 35 chars per description line
local traitor_items = {
   -- body armor
   {  id=EQUIP_ARMOR,
      type="Passive effect item",
      model="models/player/t_phoenix.mdl",
      material= mat_dir .. "icon_armor",
      name="Body Armor",
      desc="Reduces bullet damage by 20% when\nyou get hit.\n\nThe effect is automatic and always-on."
   },

   -- radar
   {  id=EQUIP_RADAR,
      type="Active use item",
      model="models/props_lab/eyescanner.mdl",
      material= mat_dir .. "icon_radar",
      name="Radar",
      desc="Allows you to scan for life signs.\n\nStarts automatic scans as soon as you\nbuy it. Configure it in Radar tab of this\nmenu."
   },

   -- Identity hider
   {  id=EQUIP_DISGUISE,
      type="Active use item",
      model="models/props/cs_office/projector_remote.mdl",
      material= mat_dir .. "icon_disguise",
      name="Disguiser",
      desc="Hides your ID info while on. Also avoids\nbeing seen last by a victim.\n\nToggle in the Disguise tab of this menu\nor press Numpad Enter."
   }
};

--- Detective stuff, some of it same as traitors
local detective_items = {
   -- radar
   {  id=EQUIP_RADAR,
      type="Active use item",
      model="models/props_lab/eyescanner.mdl",
      material= mat_dir .. "icon_radar",
      name="Radar",
      desc="Allows you to scan for life signs.\n\nStarts automatic scans as soon as you\nbuy it. Configure it in Radar tab of this\nmenu."
   }
};


local Equipment = nil
function GetEquipmentForRole(role)
   -- need to build equipment cache?
   if not Equipment then
      -- start with all the non-weapon goodies
      local tbl = {
         [ROLE_TRAITOR] = traitor_items,
         [ROLE_DETECTIVE] = detective_items
      };

      -- find buyable weapons to load info from
      for k, v in pairs(weapons.GetList()) do
         if v and v.CanBuy then
            local data = v.EquipMenuData or {}
            local base = {
               id = v.Classname,
               name = v.PrintName or "Unnamed",
               limited = v.LimitedStock,
               material = v.Icon or "VGUI/ttt/icon_id",
               -- the below should be specified in EquipMenuData, in which case
               -- these values are overridden
               type="Type not specified",
               model="models/weapons/w_bugbait.mdl", 
               desc="No description specified."
            };

            -- Force material to nil so that model key is used when we are
            -- explicitly told to do so (ie. material is false rather than nil).
            if data.modelicon then
               base.material = nil
            end

            table.Merge(base, data)

            -- add this buyable weapon to all relevant equipment tables
            for _, r in pairs(v.CanBuy) do
               table.insert(tbl[r], base)
            end
         end
      end

      Equipment = tbl
   end

   return Equipment and Equipment[role] or {}
end


local function ItemIsWeapon(item) return not tonumber(item.id) end

local color_bad = Color(220, 60, 60, 255)
local color_good = Color(0, 200, 0, 255)

-- Creates tabel of labels showing the status of ordering prerequisites
local function PreqLabels(form)
   local tbl = {}

   tbl.credits = vgui.Create("DLabel", form)
   tbl.credits:SetToolTip("Every piece of equipment you buy costs 1 credit.")
   tbl.credits.Check = function(s, sel)
                          local credits = LocalPlayer():GetCredits()
                          return credits > 0, "You have " .. credits .. " credit(s) remaining."
                       end

   tbl.owned = vgui.Create("DLabel", form)
   tbl.owned:SetToolTip("You can only buy things you do not already have.")
   tbl.owned.Check = function(s, sel)
                        if ItemIsWeapon(sel) and LocalPlayer():HasEquipmentWeapon() then
                           return false, "You are already carrying a special weapon."
                        elseif (not ItemIsWeapon(sel)) and LocalPlayer():HasEquipmentItem(sel.id) then
                           return false, "You are already carrying this item."
                        else
                           return true, "You can carry this equipment."
                        end
                     end

   tbl.bought = vgui.Create("DLabel", form)
   tbl.bought:SetToolTip("Of certain items you can only buy one per round.")
   tbl.bought.Check = function(s, sel)
                         if sel.limited and LocalPlayer():HasBought(tostring(sel.id)) then
                            return false, "This item is no longer in stock."
                         else
                            return true, "This item is in stock."
                         end
                      end

   for k, pnl in pairs(tbl) do
      pnl:SetFont("TabLarge")
      form:AddItem(pnl)
   end

   return function(selected)
             local allow = true
             for k, pnl in pairs(tbl) do
                local result, text = pnl:Check(selected)
                pnl:SetTextColor(result and color_good or color_bad)
                pnl:SetText(text)

                allow = allow and result
             end
             return allow
          end
end

local color_darkened = Color(255,255,255, 80)

local eqframe = nil
function TraitorMenuPopup()
   local ply = LocalPlayer()
   if not IsValid(ply) or not ply:IsActiveSpecial() then
      return 
   end

   -- Close any existing traitor menu
   if eqframe and ValidPanel(eqframe) then eqframe:Close() end

   local credits = ply:GetCredits()
   local can_order = credits > 0

   local dframe = vgui.Create("DFrame")
   local w, h = 500, 350
   dframe:SetSize(w, h)
   dframe:Center()
   dframe:SetTitle("Equipment")
   dframe:SetVisible(true)
   dframe:ShowCloseButton(true)
   dframe:SetMouseInputEnabled(true)
   dframe:SetDeleteOnClose(true)

   local m = 5

   local dsheet = vgui.Create("DPropertySheet", dframe)
   dsheet:SetPos(0,0)
   dsheet:StretchToParent(m,m + 25,m,m)
   local padding = dsheet:GetPadding()

   local dequip = vgui.Create("DPanel", dsheet)
   dequip:SetPaintBackground(false)
   dequip:StretchToParent(padding,padding,padding,padding)
   
   -- Determine if we already have equipment
   local owned_ids = {}
   for _, wep in pairs(ply:GetWeapons()) do
      if ValidEntity(wep) and wep.Kind == WEAPON_EQUIP then
         table.insert(owned_ids, wep:GetClass())
      end
   end

   -- Stick to one value for no equipment
   if #owned_ids == 0 then
      owned_ids = nil
   end

   --- Construct icon listing
   local dlist = vgui.Create("DPanelSelect", dequip)
   dlist:SetPos(0,0)
   dlist:SetSize(154, h - 70)
   dlist:EnableVerticalScrollbar(true)
   dlist:EnableHorizontal(true)
   dlist:SetPadding(4)

   local items = GetEquipmentForRole(ply:GetRole())

   local to_select = nil
   for k, item in pairs(items) do
      local ic = nil

      if item.material then
         ic = vgui.Create("SimpleIcon", dlist)
         ic:SetIconSize(64)
         ic:SetIcon(item.material)
      elseif item.model then
         ic = vgui.Create("SpawnIcon", dlist)
         ic:SetModel(item.model)
      else
         Warning("Equipment item does not have model or material specified: " .. tostring(item) .. "\n")
      end

      ic.item = item

      if not item.name then PrintTable(item) end

      local tip = item.name .. " (" .. item.type .. ")"
      ic:SetTooltip(tip)

      -- If we cannot order this item, darken it
      if ((not can_order) or 
          -- already owned
          table.HasValue(owned_ids, item.id) or
          (tonumber(item.id) and ply:HasEquipmentItem(tonumber(item.id))) or
          -- already carrying a weapon
          (ItemIsWeapon(item) and ply:HasEquipmentWeapon()) or
          -- already bought the item before
          (item.limited and ply:HasBought(tostring(item.id)))) then

         ic:SetIconColor(color_darkened)
      end

      dlist:AddPanel(ic)
   end

   local dlistw = 154

   local bw, bh = 100, 25

   local dih = h - bh - 25
   local diw = w - dlistw - m*5
   local dinfobg = vgui.Create("DPanel", dequip)
   dinfobg:SetPaintBackground(false)
   dinfobg:SetSize(diw, dih)
   dinfobg:SetPos(dlistw + m, 0)

   local dinfo = vgui.Create("DForm", dinfobg)
   dinfo:SetName("Item specifications")
   dinfo:SetPos(0,0)
   dinfo:SetSize(diw, 140)
   dinfo:SetPadding(8)
   dinfo:SetSpacing(5)
   dinfo:SetAutoSize(false)

   local di_name = vgui.Create("DLabel", dinfo)
   di_name:SetText("Name:")

   local di_type = vgui.Create("DLabel", dinfo)
   di_type:SetText("Type:")

   local di_desc = vgui.Create("DLabel", dinfo)
   di_desc:SetText("Description:")

   local dfields = {}
   for _, k in pairs({"name", "type", "desc"}) do
      dfields[k] = vgui.Create("DLabel", dinfo)
      dfields[k]:SetFont("TabLarge")
      dfields[k]:SetContentAlignment( 7 )
   end
   
   dinfo:AddItem(di_name, dfields.name)
   dinfo:AddItem(di_type, dfields.type)
   dinfo:AddItem(di_desc, dfields.desc)

   --dinfo:SizeToContents()

   local iw, ih = dinfo:GetSize()

   local dhelp = vgui.Create("DForm", dinfobg)
   dhelp:SetName("Ordering Status")
   dhelp:SetPos(0, ih + m)
   dhelp:SetSize(diw, dih - ih - m*2)
   dhelp:SetSpacing(4)
   dhelp:SetPadding(8)

   local update_preqs = PreqLabels(dhelp)

   dhelp:SizeToContents()

   local dconfirm = vgui.Create("DButton", dinfobg)
   dconfirm:SetPos(0, dinfobg:GetTall() - bh*2 + m)
   --dconfirm:SetPos(dlistw + m*2 + padding, h - bh - 14)
   dconfirm:SetSize(bw, bh)
   dconfirm:SetDisabled(true)
   dconfirm:SetText("Buy Equipment")


   dsheet:AddSheet("Order Equipment", dequip, "gui/silkicons/bomb", false, false, "Traitor equipment menu")

   -- Item control
   if ply:HasEquipmentItem(EQUIP_RADAR) then
      local dradar = CreateRadarMenu(dsheet, dframe)
      dsheet:AddSheet("Radar", dradar, "gui/silkicons/magnifier", false,false, "Radar control")
   end

   if ply:HasEquipmentItem(EQUIP_DISGUISE) then
      local ddisguise = CreateDisguiseMenu(dsheet)
      dsheet:AddSheet("Disguise", ddisguise, "gui/silkicons/user", false,false, "Disguise control")
   end
   
   -- Weapon/item control
   if IsValid(ply.radio) or ply:HasWeapon("weapon_ttt_radio") then
      local dradio = CreateRadioMenu(dsheet)
      dsheet:AddSheet("Radio", dradio, "gui/silkicons/sound", false,false, "Radio control")
   end


   -- couple panelselect with info
   dlist.OnActivePanelChanged = function(self, _, new)
                                   for k,v in pairs(new.item) do
                                      if dfields[k] then
                                         dfields[k]:SetText(v)
                                         dfields[k]:SizeToContents()
                                      end
                                   end

                                   -- Trying to force everything to update to
                                   -- the right size is a giant pain, so just
                                   -- force a good size.
                                   dfields.desc:SetTall(70)

                                   --dinfo:SizeToContents()
                                   --dinfo:InvalidateLayout(true)

                                   can_order = update_preqs(new.item)

                                   dconfirm:SetDisabled(not can_order)
                                end

   -- select first
   dlist:SelectPanel(to_select or dlist:GetItems()[1])

   -- prep confirm action
   dconfirm.DoClick = function()
                         local pnl = dlist.SelectedPanel
                         if not pnl or not pnl.item then return end
                         local choice = pnl.item
                         RunConsoleCommand("ttt_order_equipment", choice.id)
                         dframe:Close()
                      end

   local dcancel = vgui.Create("DButton", dframe)
   dcancel:SetPos(w - 10 - bw, h - bh - 14)
   dcancel:SetSize(bw, bh)
   dcancel:SetDisabled(false)
   dcancel:SetText("Close")
   dcancel.DoClick = function() dframe:Close() end

   dframe:MakePopup()
   dframe:SetKeyboardInputEnabled(false)

   eqframe = dframe
end

concommand.Add("ttt_cl_traitorpopup", TraitorMenuPopup)

function GM:OnContextMenuOpen()
   local r = GetRoundState()
   if r == ROUND_ACTIVE and not (LocalPlayer():GetTraitor() or LocalPlayer():GetDetective()) then 
      return
   elseif r == ROUND_POST or r == ROUND_PREP then
      CLSCORE:Reopen()
      return
   end

   RunConsoleCommand("ttt_cl_traitorpopup")
end

local function ReceiveEquipment(um)
   local ply = LocalPlayer()
   if not ValidEntity(ply) then return end

   ply.equipment_items = um:ReadChar()
end
usermessage.Hook("equipment", ReceiveEquipment)

local function ReceiveCredits(um)
   local ply = LocalPlayer()
   if not ValidEntity(ply) then return end

   ply.equipment_credits = um:ReadChar()
end
usermessage.Hook("credits", ReceiveCredits)

local r = 0
local function ReceiveBought(um)
   local ply = LocalPlayer()
   if not ValidEntity(ply) then return end

   ply.bought = {}
   local num = um:ReadShort()
   for i=1,num do
      local s = um:ReadString()
      if s != "" then
         table.insert(ply.bought, s)
      end
   end

   -- This usermessage sometimes fails to contain the last weapon that was
   -- bought, even though resending then works perfectly. Possibly a bug in
   -- bf_read. Anyway, this hack is a workaround: we just request a new umsg.
   if num != #ply.bought and r < 10 then -- r is an infinite loop guard
      RunConsoleCommand("ttt_resend_bought")
      r = r + 1
   else
      r = 0
   end
end
usermessage.Hook("bought", ReceiveBought)

