-- HUD HUD HUD

local table = table
local surface = surface
local draw = draw
local math = math

-- Fonts
surface.CreateFont("Trebuchet24", 28, 1000, true, false, "TraitorState")
surface.CreateFont("Trebuchet24", 24, 800, true, false, "TimeLeft")
surface.CreateFont("Trebuchet24", 24, 750, true, false, "HealthAmmo")

-- Color presets
local bg_colors = {
   background_main = Color(0, 0, 10, 200),
   
   noround = Color(100,100,100,200),
   traitor = Color(200, 25, 25, 200),
   innocent = Color(25, 200, 25, 200),
   detective = Color(25, 25, 200, 200)
};

local health_colors = {
   border = COLOR_WHITE,
   background = Color(100, 25, 25, 222),
   fill = Color(200, 50, 50, 250)
};

local ammo_colors = {
   border = COLOR_WHITE,
   background = Color(20, 20, 5, 222),
   fill = Color(205, 155, 0, 255)
};


-- Modified RoundedBox
local Tex_Corner8 = surface.GetTextureID( "gui/corner8" )
local function RoundedMeter( bs, x, y, w, h, color)
   surface.SetDrawColor(clr(color))
   
   surface.DrawRect( x+bs, y, w-bs*2, h )
   surface.DrawRect( x, y+bs, bs, h-bs*2 )

   surface.SetTexture( Tex_Corner8 )
   surface.DrawTexturedRectRotated( x + bs/2 , y + bs/2, bs, bs, 0 ) 
   surface.DrawTexturedRectRotated( x + bs/2 , y + h -bs/2, bs, bs, 90 )

   if w > 14 then
      surface.DrawRect( x+w-bs, y+bs, bs, h-bs*2 )
      surface.DrawTexturedRectRotated( x + w - bs/2 , y + bs/2, bs, bs, 270 )
      surface.DrawTexturedRectRotated( x + w - bs/2 , y + h - bs/2, bs, bs, 180 )
   else
      surface.DrawRect( x + math.max(w-bs, bs), y, bs/2, h )
   end

end

---- The bar painting is loosely based on:
---- http://wiki.garrysmod.com/?title=Creating_a_HUD

-- Paints a graphical meter bar
local function PaintBar(x, y, w, h, colors, value)
   -- Background
   -- slightly enlarged to make a subtle border
   draw.RoundedBox(8, x-1, y-1, w+2, h+2, colors.background)

   -- Fill
   local width = w * math.Clamp(value, 0, 1)
   
   if width > 0 then
      RoundedMeter(8, x, y, width, h, colors.fill)
   end
end

local roundstate_string = {
   [ROUND_WAIT]   = "Waiting...",
   [ROUND_PREP]   = "Preparing...",
   [ROUND_ACTIVE] = "In progress...",
   [ROUND_POST]   = "Round over"
};

function FormatRoundState() return roundstate_string[GAMEMODE.round_state] end

local role_string = {
   [ROLE_INNOCENT] = "INNOCENT",
   [ROLE_TRAITOR]  = "TRAITOR",
   [ROLE_DETECTIVE]= "DETECTIVE"
};

function RoleToString(ply) return role_string[ply:GetRole()] or "NONE" end

-- Returns player's ammo information
function GetAmmo(ply)
   local weap = ply:GetActiveWeapon()
   if not weap or not ply:Alive() then return -1 end

   local ammo_inv = weap:Ammo1()
   local ammo_clip = weap:Clip1()
   local ammo_max = weap.Primary.ClipSize

   return ammo_clip, ammo_max, ammo_inv
end

local function DrawBg(x, y, width, height, client)
   -- Traitor area sizes
   local th = 30
   local tw = 170

   -- Adjust for these
   y = y - th
   height = height + th

   -- main bg area, invariant
   -- encompasses entire area
--   surface.SetDrawColor(clr(bg_colors.background_main))
--   surface.DrawRect(x, y, width, height)
   draw.RoundedBox(8, x, y, width, height, bg_colors.background_main)
   
   -- main border, traitor based
   local col = bg_colors.innocent
   if GAMEMODE.round_state != ROUND_ACTIVE then
      col = bg_colors.noround
   elseif client:GetTraitor() then
      col = bg_colors.traitor
   elseif client:GetDetective() then
      col = bg_colors.detective
   end

   draw.RoundedBox(8, x, y, tw, th, col)
end

local function SpecHUDPaint(client)

   -- Draw round state
   local margin = 10
   local x = margin
   local height = 32
   local width = 250
   local round_y = ScrH() - height - margin

   -- move up a little on low resolutions to allow space for spectator hints
   if ScrW() < 1000 then round_y = round_y - 15 end

   local time_x = x + 170
   local time_y = round_y + 4

   draw.RoundedBox(8, x, round_y, width, height, bg_colors.background_main)
   draw.RoundedBox(8, x, round_y, time_x - x, height, bg_colors.noround)

   local text = roundstate_string[GAMEMODE.round_state]

   local spec = {text=text, font="TraitorState", pos={x+margin, round_y}, xalign=TEXT_ALIGN_LEFT, yalign=0, color=COLOR_WHITE}
   draw.Text(spec)
   draw.TextShadow(spec, 2, 255)
   
   -- Draw round time
   if GAMEMODE.round_state == ROUND_ACTIVE then
      local text = string.FormattedTime(GetGlobalFloat("ttt_round_end", 0) - CurTime(), "%02i:%02i")
      local spec = {text=text, font="TimeLeft", pos={time_x+margin, time_y}, xalign=0, yalign=0, color=COLOR_WHITE}
      draw.Text(spec)
      draw.TextShadow(spec, 2, 255)
   end

   local tgt = client:GetObserverTarget()
   if IsValid(tgt) and tgt:IsPlayer() then
      local spec = {text=tgt:Nick(), font="TimeLeft", pos={ScrW() / 2, margin}, xalign=TEXT_ALIGN_CENTER, yalign=0, color=COLOR_WHITE}
      draw.Text(spec)
      draw.TextShadow(spec, 2, 255)
   elseif IsValid(tgt) and tgt:GetNWEntity("spec_owner", nil) == client then
      local punch = client:GetNWFloat("specpunches", 0)

      width, height = 200, 25
      x = ScrW() / 2 - width/2
      --y = ScrH() - margin - height * 2
      y = margin/2 + height

      PaintBar(x, y, width, height, ammo_colors, punch)

      local spec = {text="PUNCH-O-METER", font="HealthAmmo", pos={ScrW() / 2, y}, xalign=TEXT_ALIGN_CENTER, yalign=0, color=bg_colors.background_main}
      draw.Text(spec)

      spec = {text="Move keys or jump: punch object. Crouch: leave object.", font="TabLarge", pos={ScrW() / 2, margin}, xalign=TEXT_ALIGN_CENTER}
      draw.Text(spec)

      local bonus = client:GetNWInt("bonuspunches", 0)
      if bonus != 0 then
         spec = {font="TabLarge", pos={ScrW() / 2, y*2}, xalign=TEXT_ALIGN_CENTER}

         if bonus < 0 then
            spec.text = "Your bad score lowered your punch-o-meter limit by " .. bonus .. " :("
         else
            spec.text = "Your good score increased your punch-o-meter limit by " .. bonus .. "!"
         end

         draw.Text(spec)
      end
   else
      local spec = {text="Click to spectate players, or press USE on a physics object to possess it.", font="TabLarge", pos={ScrW() / 2, margin}, xalign=TEXT_ALIGN_CENTER, yalign=0}

      draw.Text(spec)
      draw.TextShadow(spec, 2, 155)
   end
end

-- Paints player status HUD element in the bottom left
function GM:HUDPaint()
   local client = LocalPlayer()
   
   GAMEMODE:HUDDrawTargetID()
   
   if not client:Alive() or client:Team() == TEAM_SPEC then 
      SpecHUDPaint(client)

      MSTACK:Draw(client)
      return 
   end

   MSTACK:Draw(client)

   RADAR:Draw(client)
   WSWITCH:Draw(client)

   DisguisePaint(client)

   GAMEMODE:HUDDrawPickupHistory()

   -- Draw bottom left info panel (ought to move into a class/fn...)

   local margin = 10
   local width = 250
   local height = 90
   
   local x = margin
   local y = ScrH() - margin - height
   
   DrawBg(x, y, width, height, client)

   local bar_height = 25
   local bar_width = width - (margin*2)
   
   -- Draw health
   local health = math.max(0, client:Health())
   local health_y = y + margin
   PaintBar(x+margin, health_y, bar_width, bar_height, health_colors, health/100)
   local text = string.format("%i", health)
   local spec = {text=text, font="HealthAmmo", pos={bar_width, health_y}, xalign=TEXT_ALIGN_RIGHT, yalign=TEXT_ALIGN_RIGHT, color=COLOR_WHITE}
   draw.Text(spec)
   draw.TextShadow(spec, 2, 255)

   -- Ammo
   if client:GetActiveWeapon().Primary then
      local ammo_clip, ammo_max, ammo_inv = GetAmmo(client)
      if ammo_clip != -1 then
         local ammo_y = health_y + bar_height + margin
         PaintBar(x+margin, ammo_y, bar_width, bar_height, ammo_colors, ammo_clip/ammo_max)
         local text = string.format("%i + %02i", ammo_clip, ammo_inv)
         local spec = {text=text, font="HealthAmmo", pos={bar_width, ammo_y}, xalign=TEXT_ALIGN_RIGHT, yalign=TEXT_ALIGN_RIGHT, color=COLOR_WHITE}
         draw.Text(spec)
         draw.TextShadow(spec, 2, 255)
      end
   end

   -- Draw traitor state
   local traitor_y = y - 30
   local text = nil
   if GAMEMODE.round_state == ROUND_ACTIVE then
      text = role_string[client:GetRole()]
   else
      text = roundstate_string[GAMEMODE.round_state]
   end
   local spec = {text=text, font="TraitorState", pos={x+margin+73, traitor_y}, xalign=TEXT_ALIGN_CENTER,  color=COLOR_WHITE}
   draw.Text(spec)
   draw.TextShadow(spec, 2, 255)
   
   -- Draw round time
   if GAMEMODE.round_state == ROUND_ACTIVE then
      local is_haste = HasteMode()
      local is_traitor = client:IsActiveTraitor()

      local endtime = GetGlobalFloat("ttt_round_end", 0) - CurTime()

      local spec = {font="TimeLeft", color=COLOR_WHITE, pos={ x + margin + 170, traitor_y + 3}}

      if is_haste then
         local hastetime = GetGlobalFloat("ttt_haste_end", 0) - CurTime()
         if hastetime < 0 then
            if (not is_traitor) or (math.ceil(CurTime()) % 7 <= 2) then
               -- innocent or blinking "overtime"
               spec.text = "OVERTIME"
               spec.font = "Trebuchet19"
               spec.pos  = {x+margin+167, traitor_y+6}
            else
               -- traitor and not blinking "overtime" right now, so standard endtime display
               spec.text  = string.FormattedTime(math.max(0, endtime), "%02i:%02i")
               spec.color = COLOR_RED
            end
         else
            -- still in starting period
            local t = hastetime
            if is_traitor and math.ceil(CurTime()) % 6 < 2 then
               t = endtime
               spec.color = COLOR_RED
            end
            spec.text = string.FormattedTime(math.max(0, t), "%02i:%02i")
         end

         draw.Text(spec)
         draw.TextShadow(spec, 2, 255)
      else
         spec.text = string.FormattedTime(math.max(0, endtime), "%02i:%02i")

         draw.Text(spec)
         draw.TextShadow(spec, 2, 255)
      end

      if is_haste then
         local spec = {text="HASTE MODE", font="TabLarge", pos={x+margin+165, traitor_y-8}, color=COLOR_WHITE}
         draw.Text(spec)
      end
   end
end

-- Hide the standard HUD stuff
function GM:HUDShouldDraw(name)
   for k, v in pairs({"CHudHealth", "CHudBattery", "CHudAmmo", "CHudSecondaryAmmo"}) do
       if name == v then return false end
    end

    return true
end

