---- HUD stuff similar to weapon/ammo pickups but for game status messages

-- This is some of the oldest TTT code, and some of the first Lua code I ever
-- wrote. It's not the greatest.

MSTACK = {}

MSTACK.msgs = {}
MSTACK.last = 0

-- Localise some libs
local table = table
local surface = surface
local draw = draw

-- Constants for configuration
local msgfont = "DefaultBold"

local margin = 6
local msg_width = 400

local text_width = msg_width - (margin * 3) -- three margins for a little more room

local text_height = draw.GetFontHeight(msgfont)

local top_y = margin
local top_x = ScrW() - margin - msg_width

local staytime = 12
local max_items = 8

local fadein = 0.1
local fadeout = 0.6

local movespeed = 2

-- Text colors to render the messages in
local msgcolors = {
   traitor_text = COLOR_RED,
   generic_text = COLOR_WHITE,

   background = Color(0, 0, 0, 200)
};

-- Total width we take up on screen, for other elements to read
MSTACK.width = msg_width + margin

function MSTACK:AddColoredMessage(text, clr)
   local item = {}
   item.text = self:WrapText(text, text_width)
   -- Height depends on number of lines, which is equal to number of table
   -- elements of the wrapped item.text
   item.height = (#item.text * text_height) + (margin * (1 + #item.text))

   item.col = clr or msgcolors.generic_text
   item.time = CurTime()
   item.sounded = false
   item.move_y = -item.height

   -- Stagger the fading a bit
   if self.last > (item.time - 1) then
      item.time = self.last + 1 --item.time + 1
   end

   -- Insert at the top
   table.insert(self.msgs, 1, item)

   self.last = item.time
end

-- Add a given message to the stack, will be rendered in a different color if it
-- is a special traitor-only message that traitors should pay attention to.
-- Use the newer AddColoredMessage if you want special colours.
function MSTACK:AddMessage(text, traitor_only)
   self:AddColoredMessage(text, traitor_only and msgcolors.traitor_text or msgcolors.generic_text)
end

-- Oh joy, I get to write my own wrapping function. Thanks Lua!
-- Splits a string into a table of strings that are under the given width.
function MSTACK:WrapText(text, width)
   surface.SetFont(msgfont)

   -- Any wrapping required?
   local w, _ = surface.GetTextSize(text)
   if w <= width then
      return {text} -- Nope, but wrap in table for uniformity
   end
   
   local words = string.Explode(" ", text) -- No spaces means you're screwed

   local lines = {""}
   for i, wrd in pairs(words) do
      local l = #lines
      local added = lines[l] .. " " .. wrd
      w, _ = surface.GetTextSize(added)

      if w > text_width then
         -- New line needed
         table.insert(lines, wrd)
      else
         -- Safe to tack it on
         lines[l] = added
      end
   end

   -- (RANT
   --   Why do I need to count the number of lines just to access the last one?
   --   Why is there a next() but not a last() and a first()? Why do I need
   --   table.insert just to add something to the end of a table?
   --   Oh Lua, how doth thee suck? Let me count the ways...)

   return lines
end

local msg_sound = Sound("Hud.Hint")

local bg_orig = msgcolors.background
function MSTACK:Draw(client)
   if next(self.msgs) == nil then return end -- fast empty check

   local running_y = top_y
   for k, item in pairs(self.msgs) do
      if item.time < CurTime() then
         if item.sounded == false then
            client:EmitSound(msg_sound, 80, 250)
            item.sounded = true
         end

         -- Apply move effects to y
         local y = running_y + margin + item.move_y

         item.move_y = (item.move_y < 0) and item.move_y + movespeed or 0

         local delta = (item.time + staytime) - CurTime()
         delta = delta / staytime -- pct of staytime left

         -- Hurry up if we have too many
         if k >= max_items then
            delta = delta / 2
         end

         local alpha = 255
         -- These somewhat arcane delta and alpha equations are from gmod's
         -- HUDPickup stuff
         if delta > 1 - fadein then
            alpha = math.Clamp( (1.0 - delta) * (255 / fadein), 0, 255)
         elseif delta < fadeout then
            alpha = math.Clamp( delta * (255 / fadeout), 0, 255)
         end

         local height = item.height

         -- Background box
         local bg_col = Color(bg_orig.r, bg_orig.g, bg_orig.b, math.Clamp(alpha, 0, bg_orig.a))
         draw.RoundedBox(8, top_x, y, msg_width, height, bg_col)

         -- Text
         local txt_col = table.Copy(item.col)
         txt_col.a = math.Clamp(alpha, 0, txt_col.a)

         local spec = {}
         spec.font=msgfont
         spec.xalign = TEXT_ALIGN_CENTER
         spec.yalign = TEXT_ALIGN_TOP
         spec.color = txt_col

         for i = 1, #item.text do
            spec.text=item.text[i]

            local tx = top_x + (msg_width / 2)
            local ty = y + margin + (i - 1) * (text_height + margin)
            spec.pos={tx, ty}

            draw.Text(spec)
            draw.TextShadow(spec, 1, alpha)
         end

         if alpha == 0 then 
            self.msgs[k] = nil 
         end

         running_y = y + height
      end
   end
end

-- Game state message channel
local function ReceiveGameMsg(um)
   local text = um:ReadString()
   local special = um:ReadBool()

   print(text)

   MSTACK:AddMessage(text, special)
end
usermessage.Hook("game_msg", ReceiveGameMsg)

local function ReceiveCustomMsg(um)
   local text = um:ReadString()
   local clr = Color(255, 255, 255)

   clr.r = um:ReadShort()
   clr.g = um:ReadShort()
   clr.b = um:ReadShort()

   print(text)

   MSTACK:AddColoredMessage(text, clr)
end
usermessage.Hook("game_msg_color", ReceiveCustomMsg)

