-- Traitor radar rendering

RADAR = {}
RADAR.targets = {}
RADAR.enable = false
RADAR.duration = 30
RADAR.endtime = 0
RADAR.bombs = {}
RADAR.bombs_count = 0
RADAR.repeating = true
RADAR.beacons = {}
RADAR.beacons_count = 0
RADAR.samples = {}
RADAR.samples_count = 0

function RADAR:EndScan()
   self.enable = false
   self.endtime = CurTime()
end

function RADAR:Clear()
   self:EndScan()
   self.bombs = {}
   self.beacons = {}
   self.samples = {}

   self.bombs_count = 0
   self.beacons_count = 0
   self.samples_count = 0
end

function RADAR:Timeout()
   self:EndScan()

   if self.repeating and LocalPlayer() and (LocalPlayer():IsActiveTraitor() or LocalPlayer():IsActiveDetective()) then
      RunConsoleCommand("ttt_radar_scan")
   end
end

-- cache stuff we'll be drawing
function RADAR.CacheEnts()
   RADAR.beacons = {}

   for k,v in pairs(ents.FindByClass("ttt_beacon")) do
      if IsValid(v) then
         local ply = v:GetOwner()
         table.insert(RADAR.beacons, {pos=v:GetPos(), nick=(ply:IsPlayer() and ply:Nick() or "(Unknown)")})
      end
   end

   RADAR.beacons_count = #RADAR.beacons
end


function IsOffScreen(scrpos)
   return not scrpos.visible or scrpos.x < 0 or scrpos.y < 0 or scrpos.x > ScrW() or scrpos.y > ScrH()
end

local function DrawTarget(tgt, size, offset)
   local scrpos = tgt.pos:ToScreen() -- sweet
   local sz = IsOffScreen(scrpos) and size/2 or size

   scrpos.x = math.Clamp(scrpos.x, sz, ScrW() - sz)
   scrpos.y = math.Clamp(scrpos.y, sz, ScrH() - sz)

   surface.DrawTexturedRect(scrpos.x - sz, scrpos.y - sz, sz * 2, sz * 2)

   if sz == size then
      local text = math.ceil(LocalPlayer():GetPos():Distance(tgt.pos))
      local w, h = surface.GetTextSize(text)

      if offset then
         surface.SetTextPos(scrpos.x - w/2, scrpos.y + (offset * sz) - h/2)
         surface.DrawText(text)
      else
         surface.SetTextPos(scrpos.x - w/2, scrpos.y - h/2)
         surface.DrawText(text)
      end

      if tgt.t then
         text = string.FormattedTime(tgt.t - CurTime(), "%02i:%02i")
         w, h = surface.GetTextSize(text)
         
         surface.SetTextPos(scrpos.x - w / 2, scrpos.y + sz / 2)
         surface.DrawText(text)
      elseif tgt.nick then
         text = tgt.nick
         w, h = surface.GetTextSize(text)
         
         surface.SetTextPos(scrpos.x - w / 2, scrpos.y + sz / 2)
         surface.DrawText(text)
      end
   end
end

local indicator = surface.GetTextureID("effects/select_ring")
local c4warn = surface.GetTextureID("VGUI/ttt/icon_c4warn")
local det_beacon = surface.GetTextureID("VGUI/ttt/det_beacon")
local sample_scan = surface.GetTextureID("VGUI/ttt/sample_scan")
function RADAR:Draw(client)
   if not client then return end

   surface.SetFont("HudSelectionText")

   -- C4 warnings
   if self.bombs_count != 0 and client:IsActiveTraitor() then
      surface.SetTexture(c4warn)
      surface.SetTextColor(200, 55, 55, 220)
      surface.SetDrawColor(255, 255, 255, 200)
      
      for k, bomb in pairs(self.bombs) do
         DrawTarget(bomb, 24)
      end
   end

   -- Beacons
   if self.beacons_count != 0 then
      surface.SetTexture(det_beacon)
      surface.SetTextColor(55, 55, 255, 250)
      surface.SetDrawColor(255, 255, 255, 255)
      
      for k, beacon in pairs(self.beacons) do
         DrawTarget(beacon, 16, -1)
      end
   end

   -- Samples
   if self.samples_count != 0 then
      surface.SetTexture(sample_scan)
      surface.SetTextColor(200, 50, 50, 255)
      surface.SetDrawColor(255, 255, 255, 240)

      for k, sample in pairs(self.samples) do
         DrawTarget(sample, 16, 0.5)
      end
   end

   -- Player radar
   if (not self.enable) or (not client:IsActiveSpecial()) then return end

   surface.SetTexture(indicator)

   local remaining = math.max(0, RADAR.endtime - CurTime())
   local alpha = 50 + 180 * (remaining / RADAR.duration)

   local ply = nil
   for k, tgt in pairs(RADAR.targets) do
      ply = Entity(tgt.idx)
      if IsValid(ply) then
         if ply:IsTraitor() then
            surface.SetDrawColor(255, 0, 0, alpha)
            surface.SetTextColor(255, 0, 0, alpha)
         elseif ply:IsDetective() then
            surface.SetDrawColor(0, 0, 255, alpha)
            surface.SetTextColor(0, 0, 255, alpha)
         else
            surface.SetDrawColor(0, 255, 0, alpha)
            surface.SetTextColor(0, 255, 0, alpha)
         end

         DrawTarget(tgt, 32)
      end
   end

   -- Time until next scan
   surface.SetFont("TabLarge")
   surface.SetTextColor(255, 0, 0, 230)

   local text = string.FormattedTime(remaining, "Radar ready for next scan in: %02i:%02i")
   local w, h = surface.GetTextSize(text)

   surface.SetTextPos(36, ScrH() - 140 - h)
   surface.DrawText(text)
end

local function ReceiveC4Warn(um)
   local idx = um:ReadShort()
   local armed = um:ReadBool()

   if armed then
      local pos = um:ReadVector()
      local etime = um:ReadFloat()

      RADAR.bombs[idx] = {pos=pos, t=etime}
   else
      RADAR.bombs[idx] = nil
   end

   RADAR.bombs_count = table.Count(RADAR.bombs)
end
usermessage.Hook("c4_warn", ReceiveC4Warn)

local function ReceiveRadarScan(um)
   local num_targets = um:ReadChar()

   RADAR.targets = {}
   for i=1, num_targets do
      local idx = um:ReadShort()

      local pos = Vector()
      pos.x = um:ReadShort()
      pos.y = um:ReadShort()
      pos.z = um:ReadShort()

      table.insert(RADAR.targets, {idx=idx, pos=pos})
   end

   RADAR.enable = true
   RADAR.endtime = CurTime() + RADAR.duration
   
   timer.Create("radartimeout", RADAR.duration + 1, 1, RADAR.Timeout, RADAR)
end
usermessage.Hook("radar", ReceiveRadarScan)

function CreateRadarMenu(parent, frame)
   local w, h = parent:GetSize()

   local dform = vgui.Create("DForm", parent)
   dform:SetName("Scan control")
   dform:StretchToParent(0,0,0,0)
   dform:SetAutoSize(false)

   local owned = LocalPlayer():HasEquipmentItem(EQUIP_RADAR)

   if not owned then
      dform:Help("You are not carrying a radar!")
      return dform
   end

   local bw, bh = 100, 25
   local dscan = vgui.Create("DButton", dform)
   dscan:SetSize(bw, bh)
   dscan:SetText("Perform scan")
   dscan.DoClick = function(s)
                      s:SetDisabled(true)
                      RunConsoleCommand("ttt_radar_scan")
                      frame:Close()
                   end
   dform:AddItem(dscan)

   local dlabel = vgui.Create("DLabel", dform)
   dlabel:SetText("Scan results show for " .. RADAR.duration .. "s, after which the radar will have recharged and can be used again.")
   dlabel:SetWrap(true)
   dform:AddItem(dlabel)

   local dcheck = vgui.Create("DCheckBoxLabel", dform)
   dcheck:SetText("Auto-repeat scan")
   dcheck:SetIndent(5)
   dcheck:SetValue(RADAR.repeating)
   dcheck.OnChange = function(s, val)
                        RADAR.repeating = val
                     end
   dform:AddItem(dcheck)

   dform.Think = function(s)
                    if RADAR.enable or not owned then
                       dscan:SetDisabled(true)
                    else
                       dscan:SetDisabled(false)
                    end
                 end

   dform:SetVisible(true)

   return dform
end

local indicator_mat = Material("VGUI/ttt/sprite_traitor")
local indicator_col = Color(255, 255, 255, 75)

local function DrawTraitorIndicator(ply)
   if not IsValid(ply) then return end
   local client = LocalPlayer()

   if client:GetTraitor() and ply:IsActiveTraitor() then
      local pos = ply:GetPos()
      pos.z = pos.z + 74

      render.SetMaterial(indicator_mat)
      render.DrawQuadEasy(pos, client:GetForward() * -1, 8, 8, indicator_col, 180)
   end
end
-- An undocumented hook! I'm so excited!
hook.Add( "PrePlayerDraw", "TTTTraitorIndicator", DrawTraitorIndicator)

