
function GetTraitors()
   local trs = {}
   for k,v in ipairs(player.GetAll()) do
      if v:GetTraitor() then table.insert(trs, v) end
   end

   return trs
end

function CountTraitors() return #GetTraitors() end

---- Role state communication

-- Send every player their role
function SendPlayerRoles()
   for k, v in pairs(player.GetAll()) do
      umsg.Start("ttt_role", v)
      umsg.Char(v:GetRole())
      umsg.End()      
   end
end

function SendRoleListMessage(role, role_ids, ply_or_rf)
   umsg.Start("role_list", ply_or_rf)
   -- send what kind of list this is
   umsg.Char(role)

   -- list contents
   local num_ids = #role_ids
   umsg.Char(num_ids)
   for i=1, num_ids do
      umsg.Short(role_ids[i])
   end
   umsg.End()
end

function SendRoleList(role, ply_or_rf, pred)
   local role_ids = {}
   for k, v in pairs(player.GetAll()) do
      if v:IsRole(role) then
         if not pred or (pred and pred(v)) then
            table.insert(role_ids, v:EntIndex())
         end
      end
   end

   SendRoleListMessage(role, role_ids, ply_or_rf)
end

-- Tell traitors about other traitors

function SendTraitorList(ply_or_rf, pred) SendRoleList(ROLE_TRAITOR, ply_or_rf, pred) end
function SendDetectiveList(ply_or_rf) SendRoleList(ROLE_DETECTIVE, ply_or_rf) end

-- this is purely to make sure last round's traitors/dets ALWAYS get reset
-- not happy with this, but it'll do for now
function SendInnocentList(ply_or_rf)
   -- Send innocent and detectives a list of actual innocents + traitors, while
   -- sending traitors only a list of actual innocents.
   local inno_ids = {}
   local traitor_ids = {}
   for k, v in pairs(player.GetAll()) do
      if v:IsRole(ROLE_INNOCENT) then
         table.insert(inno_ids, v:EntIndex())
      elseif v:IsRole(ROLE_TRAITOR) then
         table.insert(traitor_ids, v:EntIndex())
      end
   end

   -- traitors get actual innocent, so they do not reset their traitor mates to
   -- innocence
   SendRoleListMessage(ROLE_INNOCENT, inno_ids, GetTraitorFilter())

   -- detectives and innocents get an expanded version of the truth so that they
   -- reset everyone who is not detective
   table.Add(inno_ids, traitor_ids)
   table.Shuffle(inno_ids)
   SendRoleListMessage(ROLE_INNOCENT, inno_ids, GetInnocentFilter())
end

function SendConfirmedTraitors(ply_or_rf)
   SendTraitorList(ply_or_rf, function(p) return p:GetNWBool("body_found") end)
end

function SendFullStateUpdate()
   SendPlayerRoles()
   SendInnocentList()
   SendTraitorList(GetTraitorFilter())
   SendDetectiveList()
   -- not useful to sync confirmed traitors here
end

function SendRoleReset(ply_or_rf)
   local plys = player.GetAll()
   umsg.Start("role_list", ply_or_rf)
   umsg.Char(ROLE_INNOCENT)
   umsg.Char(#plys)
   for k, v in pairs(plys) do
      umsg.Short(v:EntIndex())
   end
   umsg.End()
end

---- Console commands

function force_terror(ply)
   if server_settings.Bool("sv_cheats", 0) then
      ply:SetRole(ROLE_INNOCENT)
      ply:UnSpectate()
      ply:SetTeam(TEAM_TERROR)

      ply:StripAll()

      ply:Spawn()
      ply:PrintMessage(HUD_PRINTTALK, "You are now on the terrorist team.")

      SendFullStateUpdate()
   end
end
concommand.Add("ttt_force_terror", force_terror)

function force_traitor(ply)
   if server_settings.Bool("sv_cheats", 0) then
      ply:SetRole(ROLE_TRAITOR)

      SendFullStateUpdate()
   end
end
concommand.Add("ttt_force_traitor", force_traitor)

function force_detective(ply)
   if server_settings.Bool("sv_cheats", 0) then
      ply:SetRole(ROLE_DETECTIVE)

      SendFullStateUpdate()
   end
end
concommand.Add("ttt_force_detective", force_detective)


function force_spectate(ply, cmd, arg)
   if ValidEntity(ply) then
      if #arg == 1 and tonumber(arg[1]) == 0 then
         ply:SetForceSpec(false)
      else
         ply:Kill()
         GAMEMODE:PlayerSpawnAsSpectator(ply)
         ply:SetTeam(TEAM_SPEC)
         ply:SetRole(ROLE_INNOCENT)
         ply:SetForceSpec(true)
      end
   end
end
concommand.Add("ttt_spectate", force_spectate)

local function GetPrintFn(ply)
   if IsValid(ply) then
      return function(...)
                local t = ""
                for _, a in ipairs(arg) do
                   t = t .. "\t" .. a
                end
                ply:PrintMessage(HUD_PRINTCONSOLE, t)
             end
   else
      return print
   end
end

local function TraitorSort(a,b)
   if not IsValid(a) then return true end
   if not IsValid(b) then return false end

   if a:GetTraitor() then return true end
   if b:GetTraitor() then return false end

   return false
end

function PrintTraitors(ply)
   if not ValidEntity(ply) or ply:IsSuperAdmin() then
      local pr = GetPrintFn(ply)

      local ps = player.GetAll()
      table.sort(ps, TraitorSort)

      for _, p in pairs(ps) do
         if ValidEntity(p) then
            pr(p:GetTraitor() and "TRAITOR" or "Innocent", ":", p:Nick())
         end
      end
   end
end
concommand.Add("ttt_print_traitors", PrintTraitors)

function PrintGroups(ply)
   local pr = GetPrintFn(ply)

   pr("User", "-", "Group")
   for _, p in pairs(player.GetAll()) do
      pr(p:Nick(), "-", p:GetNWString("UserGroup"))
   end
end
concommand.Add("ttt_print_usergroups", PrintGroups)

function PrintReport(ply)
   local pr = GetPrintFn(ply)

   if not ValidEntity(ply) or ply:IsSuperAdmin() then
      for k, e in pairs(SCORE.Events) do
         if e.id == EVENT_KILL then
            if e.att.uid == -1 then
               pr("<something> killed " .. e.vic.ni .. (e.vic.tr and " [TRAITOR]" or " [inno.]"))
            else
               pr(e.att.ni .. (e.att.tr and " [TRAITOR]" or " [inno.]") .. " killed " .. e.vic.ni .. (e.vic.tr and " [TRAITOR]" or " [inno.]"))
            end
         end
      end
   else
      if ValidEntity(ply) then
         pr("You do not appear to be RCON or a superadmin!")
      end
   end
end
concommand.Add("ttt_print_adminreport", PrintReport)

-- TODO: separate all the admin stuff out sometime
local function SetAdminHighlight(ply, cmd, args)
   if not IsValid(ply) or ply:IsAdmin() then
      if #args == 1 then
         local state = tobool(args[1])
         SetGlobalBool("ttt_adminhl", state)
      end
   end
end
concommand.Add("ttt_highlight_admins", SetAdminHighlight)

local function PrintDamageLog(ply)
   local pr = GetPrintFn(ply)

   if not ValidEntity(ply) or ply:IsSuperAdmin() then
      pr("*** Damage log:\n")
      for k, txt in pairs(GAMEMODE.DamageLog) do
         pr(txt)
      end

      pr("*** Damage log end.")
   else
      if ValidEntity(ply) then
         pr("You do not appear to be RCON or a superadmin!")
      end
   end
end
concommand.Add("ttt_print_damagelog", PrintDamageLog)

local dmglog = CreateConVar("ttt_log_damage_for_console", "0")
function DamageLog(txt)
   ServerLog(txt)

   if dmglog:GetBool() then
      table.insert(GAMEMODE.DamageLog, txt)
   end
end


